# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.compute.v2 import server_ip
from openstack.tests.unit import base

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'addr': '1',
    'network_label': '2',
    'version': '4',
}


class TestServerIP(base.TestCase):
    def test_basic(self):
        sot = server_ip.ServerIP()
        self.assertEqual('addresses', sot.resources_key)
        self.assertEqual('/servers/%(server_id)s/ips', sot.base_path)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = server_ip.ServerIP(**EXAMPLE)
        self.assertEqual(EXAMPLE['addr'], sot.address)
        self.assertEqual(EXAMPLE['network_label'], sot.network_label)
        self.assertEqual(EXAMPLE['version'], sot.version)

    def test_list(self):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.get.return_value = resp
        resp.json.return_value = {
            "addresses": {
                "label1": [
                    {"version": 1, "addr": "a1"},
                    {"version": 2, "addr": "a2"},
                ],
                "label2": [
                    {"version": 3, "addr": "a3"},
                    {"version": 4, "addr": "a4"},
                ],
            }
        }

        ips = list(server_ip.ServerIP.list(sess, server_id=IDENTIFIER))

        self.assertEqual(4, len(ips))
        ips = sorted(ips, key=lambda ip: ip.version)

        self.assertIsInstance(ips[0], server_ip.ServerIP)
        self.assertEqual(ips[0].network_label, "label1")
        self.assertEqual(ips[0].address, "a1")
        self.assertEqual(ips[0].version, 1)
        self.assertIsInstance(ips[1], server_ip.ServerIP)
        self.assertEqual(ips[1].network_label, "label1")
        self.assertEqual(ips[1].address, "a2")
        self.assertEqual(ips[1].version, 2)
        self.assertIsInstance(ips[2], server_ip.ServerIP)
        self.assertEqual(ips[2].network_label, "label2")
        self.assertEqual(ips[2].address, "a3")
        self.assertEqual(ips[2].version, 3)
        self.assertIsInstance(ips[3], server_ip.ServerIP)
        self.assertEqual(ips[3].network_label, "label2")
        self.assertEqual(ips[3].address, "a4")
        self.assertEqual(ips[3].version, 4)

    def test_list_network_label(self):
        label = "label1"
        sess = mock.Mock()
        resp = mock.Mock()
        sess.get.return_value = resp
        resp.json.return_value = {
            label: [{"version": 1, "addr": "a1"}, {"version": 2, "addr": "a2"}]
        }

        ips = list(
            server_ip.ServerIP.list(
                sess, server_id=IDENTIFIER, network_label=label
            )
        )

        self.assertEqual(2, len(ips))
        ips = sorted(ips, key=lambda ip: ip.version)

        self.assertIsInstance(ips[0], server_ip.ServerIP)
        self.assertEqual(ips[0].network_label, label)
        self.assertEqual(ips[0].address, "a1")
        self.assertEqual(ips[0].version, 1)
        self.assertIsInstance(ips[1], server_ip.ServerIP)
        self.assertEqual(ips[1].network_label, label)
        self.assertEqual(ips[1].address, "a2")
        self.assertEqual(ips[1].version, 2)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Limit(resource.Resource):
    resource_key = 'limit'
    resources_key = 'limits'
    base_path = '/limits'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    commit_method = 'PATCH'
    commit_jsonpatch = True

    _query_mapping = resource.QueryParameters(
        'service_id', 'region_id', 'resource_name', 'project_id'
    )

    # Properties
    #: User-facing description of the registered_limit. *Type: string*
    description = resource.Body('description')
    #: The links for the registered_limit resource.
    links = resource.Body('links')
    #: ID of service. *Type: string*
    service_id = resource.Body('service_id')
    #: ID of region, if any. *Type: string*
    region_id = resource.Body('region_id')
    #: The resource name. *Type: string*
    resource_name = resource.Body('resource_name')
    #: The resource limit value. *Type: int*
    resource_limit = resource.Body('resource_limit')
    #: ID of project. *Type: string*
    project_id = resource.Body('project_id')

    def _prepare_request_body(self, patch, prepend_key):
        body = self._body.dirty
        if prepend_key and self.resource_key is not None:
            if patch:
                body = {self.resource_key: body}
            else:
                # Keystone support bunch create for unified limit. So the
                # request body for creating limit is a list instead of dict.
                body = {self.resources_key: [body]}
        return body



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import subnet_pool
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'address_scope_id': '1',
    'created_at': '2',
    'default_prefixlen': 3,
    'default_quota': 4,
    'description': '5',
    'id': IDENTIFIER,
    'ip_version': 6,
    'is_default': True,
    'max_prefixlen': 7,
    'min_prefixlen': 8,
    'name': '9',
    'prefixes': ['10', '11'],
    'revision_number': 12,
    'shared': True,
    'project_id': '13',
    'updated_at': '14',
}


class TestSubnetpool(base.TestCase):
    def test_basic(self):
        sot = subnet_pool.SubnetPool()
        self.assertEqual('subnetpool', sot.resource_key)
        self.assertEqual('subnetpools', sot.resources_key)
        self.assertEqual('/subnetpools', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = subnet_pool.SubnetPool(**EXAMPLE)
        self.assertEqual(EXAMPLE['address_scope_id'], sot.address_scope_id)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(
            EXAMPLE['default_prefixlen'], sot.default_prefix_length
        )
        self.assertEqual(EXAMPLE['default_quota'], sot.default_quota)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['ip_version'], sot.ip_version)
        self.assertTrue(sot.is_default)
        self.assertEqual(EXAMPLE['max_prefixlen'], sot.maximum_prefix_length)
        self.assertEqual(EXAMPLE['min_prefixlen'], sot.minimum_prefix_length)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['prefixes'], sot.prefixes)
        self.assertEqual(EXAMPLE['revision_number'], sot.revision_number)
        self.assertTrue(sot.is_shared)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack import exceptions
from openstack.tests.functional import base


class TestFlavor(base.BaseFunctionalTest):
    def setUp(self):
        super(TestFlavor, self).setUp()
        self.new_item_name = self.getUniqueString('flavor')
        self.one_flavor = list(self.conn.compute.flavors())[0]

    def test_flavors(self):
        flavors = list(self.conn.compute.flavors())
        self.assertGreater(len(flavors), 0)

        for flavor in flavors:
            self.assertIsInstance(flavor.id, str)
            self.assertIsInstance(flavor.name, str)
            self.assertIsInstance(flavor.disk, int)
            self.assertIsInstance(flavor.ram, int)
            self.assertIsInstance(flavor.vcpus, int)

    def test_find_flavors_by_id(self):
        rslt = self.conn.compute.find_flavor(self.one_flavor.id)
        self.assertEqual(rslt.id, self.one_flavor.id)

    def test_find_flavors_by_name(self):
        rslt = self.conn.compute.find_flavor(self.one_flavor.name)
        self.assertEqual(rslt.name, self.one_flavor.name)

    def test_find_flavors_no_match_ignore_true(self):
        rslt = self.conn.compute.find_flavor(
            "not a flavor", ignore_missing=True
        )
        self.assertIsNone(rslt)

    def test_find_flavors_no_match_ignore_false(self):
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.compute.find_flavor,
            "not a flavor",
            ignore_missing=False,
        )

    def test_list_flavors(self):
        pub_flavor_name = self.new_item_name + '_public'
        priv_flavor_name = self.new_item_name + '_private'
        public_kwargs = dict(
            name=pub_flavor_name, ram=1024, vcpus=2, disk=10, is_public=True
        )
        private_kwargs = dict(
            name=priv_flavor_name, ram=1024, vcpus=2, disk=10, is_public=False
        )

        # Create a public and private flavor. We expect both to be listed
        # for an operator.
        self.operator_cloud.compute.create_flavor(**public_kwargs)
        self.operator_cloud.compute.create_flavor(**private_kwargs)

        flavors = self.operator_cloud.compute.flavors()

        # Flavor list will include the standard devstack flavors. We just want
        # to make sure both of the flavors we just created are present.
        found = []
        for f in flavors:
            # extra_specs should be added within list_flavors()
            self.assertIn('extra_specs', f)
            if f['name'] in (pub_flavor_name, priv_flavor_name):
                found.append(f)
        self.assertEqual(2, len(found))

    def test_flavor_access(self):
        flavor_name = uuid.uuid4().hex
        flv = self.operator_cloud.compute.create_flavor(
            is_public=False, name=flavor_name, ram=128, vcpus=1, disk=0
        )
        self.addCleanup(self.conn.compute.delete_flavor, flv.id)
        # Validate the 'demo' user cannot see the new flavor
        flv_cmp = self.user_cloud.compute.find_flavor(flavor_name)
        self.assertIsNone(flv_cmp)

        # Validate we can see the new flavor ourselves
        flv_cmp = self.operator_cloud.compute.find_flavor(flavor_name)
        self.assertIsNotNone(flv_cmp)
        self.assertEqual(flavor_name, flv_cmp.name)

        project = self.operator_cloud.get_project('demo')
        self.assertIsNotNone(project)

        # Now give 'demo' access
        self.operator_cloud.compute.flavor_add_tenant_access(
            flv.id, project['id']
        )

        # Now see if the 'demo' user has access to it
        flv_cmp = self.user_cloud.compute.find_flavor(flavor_name)
        self.assertIsNotNone(flv_cmp)

        # Now remove 'demo' access and check we can't find it
        self.operator_cloud.compute.flavor_remove_tenant_access(
            flv.id, project['id']
        )

        flv_cmp = self.user_cloud.compute.find_flavor(flavor_name)
        self.assertIsNone(flv_cmp)

    def test_extra_props_calls(self):
        flavor_name = uuid.uuid4().hex
        flv = self.conn.compute.create_flavor(
            is_public=False, name=flavor_name, ram=128, vcpus=1, disk=0
        )
        self.addCleanup(self.conn.compute.delete_flavor, flv.id)
        # Create extra_specs
        specs = {'a': 'b'}
        self.conn.compute.create_flavor_extra_specs(flv, extra_specs=specs)
        # verify specs
        flv_cmp = self.conn.compute.fetch_flavor_extra_specs(flv)
        self.assertDictEqual(specs, flv_cmp.extra_specs)
        # update
        self.conn.compute.update_flavor_extra_specs_property(flv, 'c', 'd')
        val_cmp = self.conn.compute.get_flavor_extra_specs_property(flv, 'c')
        # fetch single prop
        self.assertEqual('d', val_cmp)
        # drop new prop
        self.conn.compute.delete_flavor_extra_specs_property(flv, 'c')
        # re-fetch and ensure prev state
        flv_cmp = self.conn.compute.fetch_flavor_extra_specs(flv)
        self.assertDictEqual(specs, flv_cmp.extra_specs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import utils


class User(resource.Resource):
    resource_key = 'user'
    resources_key = 'users'
    base_path = '/instances/%(instance_id)s/users'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True

    instance_id = resource.URI('instance_id')

    # Properties
    #: Databases the user has access to
    databases = resource.Body('databases')
    #: The name of the user
    name = resource.Body('name', alternate_id=True)
    #: The password of the user
    password = resource.Body('password')

    def _prepare_request(
        self, requires_id=True, prepend_key=True, base_path=None, **kwargs
    ):
        """Prepare a request for the database service's create call

        User.create calls require the resources_key.
        The base_prepare_request would insert the resource_key (singular)
        """
        body = {self.resources_key: self._body.dirty}

        if base_path is None:
            base_path = self.base_path

        uri = base_path % self._uri.attributes
        uri = utils.urljoin(uri, self.id)

        return resource._Request(uri, body, None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ServiceProfile(resource.Resource):
    resource_key = 'service_profile'
    resources_key = 'service_profiles'
    base_path = '/service_profiles'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'description',
        'driver',
        'project_id',
        is_enabled='enabled',
    )
    # Properties
    #: Description of the service flavor profile.
    description = resource.Body('description')
    #: Provider driver for the service flavor profile
    driver = resource.Body('driver')
    #: Sets enabled flag
    is_enabled = resource.Body('enabled', type=bool)
    #: Metainformation of the service flavor profile
    meta_info = resource.Body('metainfo')
    #: The owner project ID
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)



# Copyright(c) 2022 Inspur
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack.instance_ha.v1 import vmove
from openstack.tests.unit import base

FAKE_ID = "1"
FAKE_UUID = "16a7c91f-8342-49a7-c731-3a632293f845"
FAKE_NOTIFICATION_ID = "a0e70d3a-b3a2-4616-b65d-a7c03a2c85fc"
FAKE_SERVER_ID = "1c2f1795-ce78-4d4c-afd0-ce141fdb3952"

VMOVE = {
    'id': FAKE_ID,
    'uuid': FAKE_UUID,
    'notification_id': FAKE_NOTIFICATION_ID,
    'created_at': "2023-01-28T14:55:26.000000",
    'updated_at': "2023-01-28T14:55:31.000000",
    'server_id': FAKE_SERVER_ID,
    'server_name': 'vm1',
    'source_host': 'host1',
    'dest_host': 'host2',
    'start_time': "2023-01-28T14:55:27.000000",
    'end_time': "2023-01-28T14:55:31.000000",
    'status': 'succeeded',
    'type': 'evacuation',
    'message': None,
}


class TestVMove(base.TestCase):
    def test_basic(self):
        sot = vmove.VMove(VMOVE)
        self.assertEqual("vmove", sot.resource_key)
        self.assertEqual("vmoves", sot.resources_key)
        self.assertEqual(
            "/notifications/%(notification_id)s/vmoves", sot.base_path
        )
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)

        self.assertDictEqual(
            {
                "status": "status",
                "type": "type",
                "limit": "limit",
                "marker": "marker",
                "sort_dir": "sort_dir",
                "sort_key": "sort_key",
            },
            sot._query_mapping._mapping,
        )

    def test_create(self):
        sot = vmove.VMove(**VMOVE)
        self.assertEqual(VMOVE["id"], sot.id)
        self.assertEqual(VMOVE["uuid"], sot.uuid)
        self.assertEqual(VMOVE["notification_id"], sot.notification_id)
        self.assertEqual(VMOVE["created_at"], sot.created_at)
        self.assertEqual(VMOVE["updated_at"], sot.updated_at)
        self.assertEqual(VMOVE["server_id"], sot.server_id)
        self.assertEqual(VMOVE["server_name"], sot.server_name)
        self.assertEqual(VMOVE["source_host"], sot.source_host)
        self.assertEqual(VMOVE["dest_host"], sot.dest_host)
        self.assertEqual(VMOVE["start_time"], sot.start_time)
        self.assertEqual(VMOVE["end_time"], sot.end_time)
        self.assertEqual(VMOVE["status"], sot.status)
        self.assertEqual(VMOVE["type"], sot.type)
        self.assertEqual(VMOVE["message"], sot.message)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import openstack.cloud
from openstack.tests.unit import base


class TestOpenStackCloudOperatorNoAuth(base.TestCase):
    def setUp(self):
        """Setup Noauth OpenStackCloud tests

        Setup the test to utilize no authentication and an endpoint
        URL in the auth data.  This is permits testing of the basic
        mechanism that enables Ironic noauth mode to be utilized with
        Shade.

        Uses base.TestCase instead of IronicTestCase because
        we need to do completely different things with discovery.
        """
        super(TestOpenStackCloudOperatorNoAuth, self).setUp()
        # By clearing the URI registry, we remove all calls to a keystone
        # catalog or getting a token
        self._uri_registry.clear()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        service_type='baremetal', base_url_append='v1'
                    ),
                    json={
                        'id': 'v1',
                        'links': [
                            {
                                "href": "https://baremetal.example.com/v1",
                                "rel": "self",
                            }
                        ],
                    },
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        service_type='baremetal',
                        base_url_append='v1',
                        resource='nodes',
                    ),
                    json={'nodes': []},
                ),
            ]
        )

    def test_ironic_noauth_none_auth_type(self):
        """Test noauth selection for Ironic in OpenStackCloud

        The new way of doing this is with the keystoneauth none plugin.
        """
        # NOTE(TheJulia): When we are using the python-ironicclient
        # library, the library will automatically prepend the URI path
        # with 'v1'. As such, since we are overriding the endpoint,
        # we must explicitly do the same as we move away from the
        # client library.
        self.cloud_noauth = openstack.connect(
            auth_type='none',
            baremetal_endpoint_override="https://baremetal.example.com/v1",
        )

        self.cloud_noauth.list_machines()

        self.assert_calls()

    def test_ironic_noauth_auth_endpoint(self):
        """Test noauth selection for Ironic in OpenStackCloud

        Sometimes people also write clouds.yaml files that look like this:

        ::
          clouds:
            bifrost:
              auth_type: "none"
              endpoint: https://baremetal.example.com
        """
        self.cloud_noauth = openstack.connect(
            auth_type='none',
            endpoint='https://baremetal.example.com/v1',
        )

        self.cloud_noauth.list_machines()

        self.assert_calls()

    def test_ironic_noauth_admin_token_auth_type(self):
        """Test noauth selection for Ironic in OpenStackCloud

        The old way of doing this was to abuse admin_token.
        """
        self.cloud_noauth = openstack.connect(
            auth_type='admin_token',
            auth=dict(
                endpoint='https://baremetal.example.com/v1', token='ignored'
            ),
        )

        self.cloud_noauth.list_machines()

        self.assert_calls()


class TestOpenStackCloudOperatorNoAuthUnversioned(base.TestCase):
    def setUp(self):
        """Setup Noauth OpenStackCloud tests for unversioned endpoints

        Setup the test to utilize no authentication and an endpoint
        URL in the auth data.  This is permits testing of the basic
        mechanism that enables Ironic noauth mode to be utilized with
        Shade.

        Uses base.TestCase instead of IronicTestCase because
        we need to do completely different things with discovery.
        """
        super(TestOpenStackCloudOperatorNoAuthUnversioned, self).setUp()
        # By clearing the URI registry, we remove all calls to a keystone
        # catalog or getting a token
        self._uri_registry.clear()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri='https://baremetal.example.com/',
                    json={
                        "default_version": {
                            "status": "CURRENT",
                            "min_version": "1.1",
                            "version": "1.46",
                            "id": "v1",
                            "links": [
                                {
                                    "href": "https://baremetal.example.com/v1",
                                    "rel": "self",
                                }
                            ],
                        },
                        "versions": [
                            {
                                "status": "CURRENT",
                                "min_version": "1.1",
                                "version": "1.46",
                                "id": "v1",
                                "links": [
                                    {
                                        "href": "https://baremetal.example.com/v1",  # noqa: E501
                                        "rel": "self",
                                    }
                                ],
                            }
                        ],
                        "name": "OpenStack Ironic API",
                        "description": "Ironic is an OpenStack project.",
                    },
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        service_type='baremetal', base_url_append='v1'
                    ),
                    json={
                        "media_types": [
                            {
                                "base": "application/json",
                                "type": "application/vnd.openstack.ironic.v1+json",  # noqa: E501
                            }
                        ],
                        "links": [
                            {
                                "href": "https://baremetal.example.com/v1",
                                "rel": "self",
                            }
                        ],
                        "ports": [
                            {
                                "href": "https://baremetal.example.com/v1/ports/",  # noqa: E501
                                "rel": "self",
                            },
                            {
                                "href": "https://baremetal.example.com/ports/",
                                "rel": "bookmark",
                            },
                        ],
                        "nodes": [
                            {
                                "href": "https://baremetal.example.com/v1/nodes/",  # noqa: E501
                                "rel": "self",
                            },
                            {
                                "href": "https://baremetal.example.com/nodes/",
                                "rel": "bookmark",
                            },
                        ],
                        "id": "v1",
                    },
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        service_type='baremetal',
                        base_url_append='v1',
                        resource='nodes',
                    ),
                    json={'nodes': []},
                ),
            ]
        )

    def test_ironic_noauth_none_auth_type(self):
        """Test noauth selection for Ironic in OpenStackCloud

        The new way of doing this is with the keystoneauth none plugin.
        """
        # NOTE(TheJulia): When we are using the python-ironicclient
        # library, the library will automatically prepend the URI path
        # with 'v1'. As such, since we are overriding the endpoint,
        # we must explicitly do the same as we move away from the
        # client library.
        self.cloud_noauth = openstack.connect(
            auth_type='none',
            baremetal_endpoint_override="https://baremetal.example.com",
        )

        self.cloud_noauth.list_machines()

        self.assert_calls()

    def test_ironic_noauth_auth_endpoint(self):
        """Test noauth selection for Ironic in OpenStackCloud

        Sometimes people also write clouds.yaml files that look like this:

        ::
          clouds:
            bifrost:
              auth_type: "none"
              endpoint: https://baremetal.example.com
        """
        self.cloud_noauth = openstack.connect(
            auth_type='none',
            endpoint='https://baremetal.example.com/',
        )

        self.cloud_noauth.list_machines()

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from keystoneauth1 import adapter


from openstack.block_storage.v2 import type
from openstack.tests.unit import base


FAKE_ID = "6685584b-1eac-4da6-b5c3-555430cf68ff"
TYPE = {"extra_specs": {"capabilities": "gpu"}, "id": FAKE_ID, "name": "SSD"}


class TestType(base.TestCase):
    def setUp(self):
        super(TestType, self).setUp()
        self.extra_specs_result = {"extra_specs": {"go": "cubs", "boo": "sox"}}
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.status_code = 200
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = '3.0'
        self.sess.post = mock.Mock(return_value=self.resp)
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        sot = type.Type(**TYPE)
        self.assertEqual("volume_type", sot.resource_key)
        self.assertEqual("volume_types", sot.resources_key)
        self.assertEqual("/types", sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_commit)

    def test_new(self):
        sot = type.Type.new(id=FAKE_ID)
        self.assertEqual(FAKE_ID, sot.id)

    def test_create(self):
        sot = type.Type(**TYPE)
        self.assertEqual(TYPE["id"], sot.id)
        self.assertEqual(TYPE["extra_specs"], sot.extra_specs)
        self.assertEqual(TYPE["name"], sot.name)

    def test_get_private_access(self):
        sot = type.Type(**TYPE)

        response = mock.Mock()
        response.status_code = 200
        response.body = {
            "volume_type_access": [{"project_id": "a", "volume_type_id": "b"}]
        }
        response.json = mock.Mock(return_value=response.body)
        self.sess.get = mock.Mock(return_value=response)

        self.assertEqual(
            response.body["volume_type_access"],
            sot.get_private_access(self.sess),
        )

        self.sess.get.assert_called_with(
            "types/%s/os-volume-type-access" % sot.id
        )

    def test_add_private_access(self):
        sot = type.Type(**TYPE)

        self.assertIsNone(sot.add_private_access(self.sess, "a"))

        url = "types/%s/action" % sot.id
        body = {"addProjectAccess": {"project": "a"}}
        self.sess.post.assert_called_with(url, json=body)

    def test_remove_private_access(self):
        sot = type.Type(**TYPE)

        self.assertIsNone(sot.remove_private_access(self.sess, "a"))

        url = "types/%s/action" % sot.id
        body = {"removeProjectAccess": {"project": "a"}}
        self.sess.post.assert_called_with(url, json=body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class AbsoluteLimits(resource.Resource):
    _max_microversion = '2.57'

    #: The number of key-value pairs that can be set as image metadata.
    image_meta = resource.Body("maxImageMeta", aka="max_image_meta")
    #: The maximum number of personality contents that can be supplied.
    personality = resource.Body("maxPersonality", deprecated=True)
    #: The maximum size, in bytes, of a personality.
    personality_size = resource.Body("maxPersonalitySize", deprecated=True)
    #: The maximum amount of security group rules allowed.
    security_group_rules = resource.Body(
        "maxSecurityGroupRules", aka="max_security_group_rules"
    )
    #: The maximum amount of security groups allowed.
    security_groups = resource.Body(
        "maxSecurityGroups", aka="max_security_groups"
    )
    #: The amount of security groups currently in use.
    security_groups_used = resource.Body(
        "totalSecurityGroupsUsed", aka="total_security_groups_used"
    )
    #: The number of key-value pairs that can be set as server metadata.
    server_meta = resource.Body("maxServerMeta", aka="max_server_meta")
    #: The maximum amount of cores.
    total_cores = resource.Body("maxTotalCores", aka="max_total_cores")
    #: The amount of cores currently in use.
    total_cores_used = resource.Body("totalCoresUsed", aka="total_cores_used")
    #: The maximum amount of floating IPs.
    floating_ips = resource.Body(
        "maxTotalFloatingIps", aka="max_total_floating_ips"
    )
    #: The amount of floating IPs currently in use.
    floating_ips_used = resource.Body(
        "totalFloatingIpsUsed", aka="total_floating_ips_used"
    )
    #: The maximum amount of instances.
    instances = resource.Body("maxTotalInstances", aka="max_total_instances")
    #: The amount of instances currently in use.
    instances_used = resource.Body(
        "totalInstancesUsed", aka="total_instances_used"
    )
    #: The maximum amount of keypairs.
    keypairs = resource.Body("maxTotalKeypairs", aka="max_total_keypairs")
    #: The maximum RAM size in megabytes.
    total_ram = resource.Body("maxTotalRAMSize", aka="max_total_ram_size")
    #: The RAM size in megabytes currently in use.
    total_ram_used = resource.Body("totalRAMUsed", aka="total_ram_used")
    #: The maximum amount of server groups.
    server_groups = resource.Body("maxServerGroups", aka="max_server_groups")
    #: The amount of server groups currently in use.
    server_groups_used = resource.Body(
        "totalServerGroupsUsed", aka="total_server_groups_used"
    )
    #: The maximum number of members in a server group.
    server_group_members = resource.Body(
        "maxServerGroupMembers", aka="max_server_group_members"
    )


class RateLimit(resource.Resource):
    # TODO(mordred) Make a resource type for the contents of limit and add
    # it to list_type here.
    #: A list of the specific limits that apply to the ``regex`` and ``uri``.
    limits = resource.Body("limit", type=list)
    #: A regex representing which routes this rate limit applies to.
    regex = resource.Body("regex")
    #: A URI representing which routes this rate limit applies to.
    uri = resource.Body("uri")


class Limits(resource.Resource):
    base_path = "/limits"
    resource_key = "limits"

    allow_fetch = True

    _query_mapping = resource.QueryParameters('tenant_id')

    absolute = resource.Body("absolute", type=AbsoluteLimits)
    rate = resource.Body("rate", type=list, list_type=RateLimit)

    def fetch(
        self,
        session,
        requires_id=False,
        error_message=None,
        base_path=None,
        skip_cache=False,
        **params
    ):
        """Get the Limits resource.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :returns: A Limits instance
        :rtype: :class:`~openstack.compute.v2.limits.Limits`
        """
        # TODO(mordred) We shouldn't have to subclass just to declare
        # requires_id = False.
        return super(Limits, self).fetch(
            session=session,
            requires_id=requires_id,
            error_message=error_message,
            base_path=base_path,
            skip_cache=skip_cache,
            **params
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from hashlib import sha1
import random
import string
import tempfile
import time
from unittest import mock

import requests_mock
from testscenarios import load_tests_apply_scenarios as load_tests  # noqa

from openstack.object_store.v1 import account
from openstack.object_store.v1 import container
from openstack.object_store.v1 import obj
from openstack.tests.unit.cloud import test_object as base_test_object
from openstack.tests.unit import test_proxy_base


class FakeResponse:
    def __init__(self, response, status_code=200, headers=None):
        self.body = response
        self.status_code = status_code
        self.headers = headers if headers else {}

    def json(self):
        return self.body


class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
    kwargs_to_path_args = False

    def setUp(self):
        super(TestObjectStoreProxy, self).setUp()
        self.proxy = self.cloud.object_store
        self.container = self.getUniqueString()
        self.endpoint = self.cloud.object_store.get_endpoint() + '/'
        self.container_endpoint = '{endpoint}{container}'.format(
            endpoint=self.endpoint, container=self.container
        )

    def test_account_metadata_get(self):
        self.verify_head(
            self.proxy.get_account_metadata, account.Account, method_args=[]
        )

    def test_container_metadata_get(self):
        self.verify_head(
            self.proxy.get_container_metadata,
            container.Container,
            method_args=["container"],
        )

    def test_container_delete(self):
        self.verify_delete(
            self.proxy.delete_container, container.Container, False
        )

    def test_container_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_container, container.Container, True
        )

    def test_container_create_attrs(self):
        self.verify_create(
            self.proxy.create_container,
            container.Container,
            method_args=['container_name'],
            expected_args=[],
            expected_kwargs={'name': 'container_name', "x": 1, "y": 2, "z": 3},
        )

    def test_object_metadata_get(self):
        self._verify(
            "openstack.proxy.Proxy._head",
            self.proxy.get_object_metadata,
            method_args=['object'],
            method_kwargs={'container': 'container'},
            expected_args=[obj.Object, 'object'],
            expected_kwargs={'container': 'container'},
        )

    def _test_object_delete(self, ignore):
        expected_kwargs = {
            "ignore_missing": ignore,
            "container": "name",
        }

        self._verify(
            "openstack.proxy.Proxy._delete",
            self.proxy.delete_object,
            method_args=["resource"],
            method_kwargs=expected_kwargs,
            expected_args=[obj.Object, "resource"],
            expected_kwargs=expected_kwargs,
        )

    def test_object_delete(self):
        self._test_object_delete(False)

    def test_object_delete_ignore(self):
        self._test_object_delete(True)

    def test_object_create_attrs(self):
        kwargs = {
            "name": "test",
            "data": "data",
            "container": "name",
            "metadata": {},
        }

        self._verify(
            "openstack.proxy.Proxy._create",
            self.proxy.upload_object,
            method_kwargs=kwargs,
            expected_args=[obj.Object],
            expected_kwargs=kwargs,
        )

    def test_object_create_no_container(self):
        self.assertRaises(TypeError, self.proxy.upload_object)

    def test_object_get(self):
        with requests_mock.Mocker() as m:
            m.get("%scontainer/object" % self.endpoint, text="data")
            res = self.proxy.get_object("object", container="container")
            self.assertIsNone(res.data)

    def test_object_get_write_file(self):
        with requests_mock.Mocker() as m:
            m.get("%scontainer/object" % self.endpoint, text="data")
            with tempfile.NamedTemporaryFile() as f:
                self.proxy.get_object(
                    "object", container="container", outfile=f.name
                )
                dt = open(f.name).read()
                self.assertEqual(dt, "data")

    def test_object_get_remember_content(self):
        with requests_mock.Mocker() as m:
            m.get("%scontainer/object" % self.endpoint, text="data")
            res = self.proxy.get_object(
                "object", container="container", remember_content=True
            )
            self.assertEqual(res.data, "data")

    def test_set_temp_url_key(self):
        key = 'super-secure-key'

        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.endpoint,
                    status_code=204,
                    validate=dict(
                        headers={'x-account-meta-temp-url-key': key}
                    ),
                ),
                dict(
                    method='HEAD',
                    uri=self.endpoint,
                    headers={'x-account-meta-temp-url-key': key},
                ),
            ]
        )
        self.proxy.set_account_temp_url_key(key)
        self.assert_calls()

    def test_set_account_temp_url_key_second(self):
        key = 'super-secure-key'

        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.endpoint,
                    status_code=204,
                    validate=dict(
                        headers={'x-account-meta-temp-url-key-2': key}
                    ),
                ),
                dict(
                    method='HEAD',
                    uri=self.endpoint,
                    headers={'x-account-meta-temp-url-key-2': key},
                ),
            ]
        )
        self.proxy.set_account_temp_url_key(key, secondary=True)
        self.assert_calls()

    def test_set_container_temp_url_key(self):
        key = 'super-secure-key'

        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.container_endpoint,
                    status_code=204,
                    validate=dict(
                        headers={'x-container-meta-temp-url-key': key}
                    ),
                ),
                dict(
                    method='HEAD',
                    uri=self.container_endpoint,
                    headers={'x-container-meta-temp-url-key': key},
                ),
            ]
        )
        self.proxy.set_container_temp_url_key(self.container, key)
        self.assert_calls()

    def test_set_container_temp_url_key_second(self):
        key = 'super-secure-key'

        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.container_endpoint,
                    status_code=204,
                    validate=dict(
                        headers={'x-container-meta-temp-url-key-2': key}
                    ),
                ),
                dict(
                    method='HEAD',
                    uri=self.container_endpoint,
                    headers={'x-container-meta-temp-url-key-2': key},
                ),
            ]
        )
        self.proxy.set_container_temp_url_key(
            self.container, key, secondary=True
        )
        self.assert_calls()

    def test_copy_object(self):
        self.assertRaises(NotImplementedError, self.proxy.copy_object)

    def test_file_segment(self):
        file_size = 4200
        content = ''.join(
            random.choice(string.ascii_uppercase + string.digits)
            for _ in range(file_size)
        ).encode('latin-1')
        self.imagefile = tempfile.NamedTemporaryFile(delete=False)
        self.imagefile.write(content)
        self.imagefile.close()

        segments = self.proxy._get_file_segments(
            endpoint='test_container/test_image',
            filename=self.imagefile.name,
            file_size=file_size,
            segment_size=1000,
        )
        self.assertEqual(len(segments), 5)
        segment_content = b''
        for index, (name, segment) in enumerate(segments.items()):
            self.assertEqual(
                'test_container/test_image/{index:0>6}'.format(index=index),
                name,
            )
            segment_content += segment.read()
        self.assertEqual(content, segment_content)


class TestDownloadObject(base_test_object.BaseTestObject):
    def setUp(self):
        super(TestDownloadObject, self).setUp()
        self.the_data = b'test body'
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.object_endpoint,
                    headers={
                        'Content-Length': str(len(self.the_data)),
                        'Content-Type': 'application/octet-stream',
                        'Accept-Ranges': 'bytes',
                        'Last-Modified': 'Thu, 15 Dec 2016 13:34:14 GMT',
                        'Etag': '"b5c454b44fbd5344793e3fb7e3850768"',
                        'X-Timestamp': '1481808853.65009',
                        'X-Trans-Id': 'tx68c2a2278f0c469bb6de1-005857ed80dfw1',
                        'Date': 'Mon, 19 Dec 2016 14:24:00 GMT',
                        'X-Static-Large-Object': 'True',
                        'X-Object-Meta-Mtime': '1481513709.168512',
                    },
                    content=self.the_data,
                )
            ]
        )

    def test_download(self):
        data = self.cloud.object_store.download_object(
            self.object, container=self.container
        )

        self.assertEqual(data, self.the_data)
        self.assert_calls()

    def test_stream(self):
        chunk_size = 2
        for index, chunk in enumerate(
            self.cloud.object_store.stream_object(
                self.object, container=self.container, chunk_size=chunk_size
            )
        ):
            chunk_len = len(chunk)
            start = index * chunk_size
            end = start + chunk_len
            self.assertLessEqual(chunk_len, chunk_size)
            self.assertEqual(chunk, self.the_data[start:end])
        self.assert_calls()


class TestExtractName(TestObjectStoreProxy):
    scenarios = [
        ('discovery', dict(url='/', parts=['account'])),
        ('endpoints', dict(url='/endpoints', parts=['endpoints'])),
        (
            'container',
            dict(url='/AUTH_123/container_name', parts=['container']),
        ),
        ('object', dict(url='/container_name/object_name', parts=['object'])),
        (
            'object_long',
            dict(
                url='/v1/AUTH_123/cnt/path/deep/object_name', parts=['object']
            ),
        ),
    ]

    def test_extract_name(self):
        results = self.proxy._extract_name(self.url, project_id='123')
        self.assertEqual(self.parts, results)


class TestTempURL(TestObjectStoreProxy):
    expires_iso8601_format = '%Y-%m-%dT%H:%M:%SZ'
    short_expires_iso8601_format = '%Y-%m-%d'
    time_errmsg = (
        'time must either be a whole number or in specific ISO 8601 format.'
    )
    path_errmsg = 'path must be full path to an object e.g. /v1/a/c/o'
    url = '/v1/AUTH_account/c/o'
    seconds = 3600
    key = 'correcthorsebatterystaple'
    method = 'GET'
    expected_url = url + (
        '?temp_url_sig=temp_url_signature&temp_url_expires=1400003600'
    )
    expected_body = '\n'.join(
        [
            method,
            '1400003600',
            url,
        ]
    ).encode('utf-8')

    @mock.patch('hmac.HMAC')
    @mock.patch('time.time', return_value=1400000000)
    def test_generate_temp_url(self, time_mock, hmac_mock):
        hmac_mock().hexdigest.return_value = 'temp_url_signature'
        url = self.proxy.generate_temp_url(
            self.url, self.seconds, self.method, temp_url_key=self.key
        )
        key = self.key
        if not isinstance(key, bytes):
            key = key.encode('utf-8')
        self.assertEqual(url, self.expected_url)
        self.assertEqual(
            hmac_mock.mock_calls,
            [
                mock.call(),
                mock.call(key, self.expected_body, sha1),
                mock.call().hexdigest(),
            ],
        )
        self.assertIsInstance(url, type(self.url))

    @mock.patch('hmac.HMAC')
    @mock.patch('time.time', return_value=1400000000)
    def test_generate_temp_url_ip_range(self, time_mock, hmac_mock):
        hmac_mock().hexdigest.return_value = 'temp_url_signature'
        ip_ranges = [
            '1.2.3.4',
            '1.2.3.4/24',
            '2001:db8::',
            b'1.2.3.4',
            b'1.2.3.4/24',
            b'2001:db8::',
        ]
        path = '/v1/AUTH_account/c/o/'
        expected_url = path + (
            '?temp_url_sig=temp_url_signature'
            '&temp_url_expires=1400003600'
            '&temp_url_ip_range='
        )
        for ip_range in ip_ranges:
            hmac_mock.reset_mock()
            url = self.proxy.generate_temp_url(
                path,
                self.seconds,
                self.method,
                temp_url_key=self.key,
                ip_range=ip_range,
            )
            key = self.key
            if not isinstance(key, bytes):
                key = key.encode('utf-8')

            if isinstance(ip_range, bytes):
                ip_range_expected_url = expected_url + ip_range.decode('utf-8')
                expected_body = '\n'.join(
                    [
                        'ip=' + ip_range.decode('utf-8'),
                        self.method,
                        '1400003600',
                        path,
                    ]
                ).encode('utf-8')
            else:
                ip_range_expected_url = expected_url + ip_range
                expected_body = '\n'.join(
                    [
                        'ip=' + ip_range,
                        self.method,
                        '1400003600',
                        path,
                    ]
                ).encode('utf-8')

            self.assertEqual(url, ip_range_expected_url)

            self.assertEqual(
                hmac_mock.mock_calls,
                [
                    mock.call(key, expected_body, sha1),
                    mock.call().hexdigest(),
                ],
            )
            self.assertIsInstance(url, type(path))

    @mock.patch('hmac.HMAC')
    def test_generate_temp_url_iso8601_argument(self, hmac_mock):
        hmac_mock().hexdigest.return_value = 'temp_url_signature'
        url = self.proxy.generate_temp_url(
            self.url,
            '2014-05-13T17:53:20Z',
            self.method,
            temp_url_key=self.key,
        )
        self.assertEqual(url, self.expected_url)

        # Don't care about absolute arg.
        url = self.proxy.generate_temp_url(
            self.url,
            '2014-05-13T17:53:20Z',
            self.method,
            temp_url_key=self.key,
            absolute=True,
        )
        self.assertEqual(url, self.expected_url)

        lt = time.localtime()
        expires = time.strftime(self.expires_iso8601_format[:-1], lt)

        if not isinstance(self.expected_url, str):
            expected_url = self.expected_url.replace(
                b'1400003600',
                bytes(str(int(time.mktime(lt))), encoding='ascii'),
            )
        else:
            expected_url = self.expected_url.replace(
                '1400003600', str(int(time.mktime(lt)))
            )
        url = self.proxy.generate_temp_url(
            self.url, expires, self.method, temp_url_key=self.key
        )
        self.assertEqual(url, expected_url)

        expires = time.strftime(self.short_expires_iso8601_format, lt)
        lt = time.strptime(expires, self.short_expires_iso8601_format)

        if not isinstance(self.expected_url, str):
            expected_url = self.expected_url.replace(
                b'1400003600',
                bytes(str(int(time.mktime(lt))), encoding='ascii'),
            )
        else:
            expected_url = self.expected_url.replace(
                '1400003600', str(int(time.mktime(lt)))
            )
        url = self.proxy.generate_temp_url(
            self.url, expires, self.method, temp_url_key=self.key
        )
        self.assertEqual(url, expected_url)

    @mock.patch('hmac.HMAC')
    @mock.patch('time.time', return_value=1400000000)
    def test_generate_temp_url_iso8601_output(self, time_mock, hmac_mock):
        hmac_mock().hexdigest.return_value = 'temp_url_signature'
        url = self.proxy.generate_temp_url(
            self.url,
            self.seconds,
            self.method,
            temp_url_key=self.key,
            iso8601=True,
        )
        key = self.key
        if not isinstance(key, bytes):
            key = key.encode('utf-8')

        expires = time.strftime(
            self.expires_iso8601_format, time.gmtime(1400003600)
        )
        if not isinstance(self.url, str):
            self.assertTrue(url.endswith(bytes(expires, 'utf-8')))
        else:
            self.assertTrue(url.endswith(expires))
        self.assertEqual(
            hmac_mock.mock_calls,
            [
                mock.call(),
                mock.call(key, self.expected_body, sha1),
                mock.call().hexdigest(),
            ],
        )
        self.assertIsInstance(url, type(self.url))

    @mock.patch('hmac.HMAC')
    @mock.patch('time.time', return_value=1400000000)
    def test_generate_temp_url_prefix(self, time_mock, hmac_mock):
        hmac_mock().hexdigest.return_value = 'temp_url_signature'
        prefixes = ['', 'o', 'p0/p1/']
        for p in prefixes:
            hmac_mock.reset_mock()
            path = '/v1/AUTH_account/c/' + p
            expected_url = path + (
                '?temp_url_sig=temp_url_signature'
                '&temp_url_expires=1400003600'
                '&temp_url_prefix=' + p
            )
            expected_body = '\n'.join(
                [
                    self.method,
                    '1400003600',
                    'prefix:' + path,
                ]
            ).encode('utf-8')
            url = self.proxy.generate_temp_url(
                path,
                self.seconds,
                self.method,
                prefix=True,
                temp_url_key=self.key,
            )
            key = self.key
            if not isinstance(key, bytes):
                key = key.encode('utf-8')
            self.assertEqual(url, expected_url)
            self.assertEqual(
                hmac_mock.mock_calls,
                [
                    mock.call(key, expected_body, sha1),
                    mock.call().hexdigest(),
                ],
            )

            self.assertIsInstance(url, type(path))

    def test_generate_temp_url_invalid_path(self):
        self.assertRaisesRegex(
            ValueError,
            'path must be representable as UTF-8',
            self.proxy.generate_temp_url,
            b'/v1/a/c/\xff',
            self.seconds,
            self.method,
            temp_url_key=self.key,
        )

    @mock.patch('hmac.HMAC.hexdigest', return_value="temp_url_signature")
    def test_generate_absolute_expiry_temp_url(self, hmac_mock):
        if isinstance(self.expected_url, bytes):
            expected_url = self.expected_url.replace(
                b'1400003600', b'2146636800'
            )
        else:
            expected_url = self.expected_url.replace(
                u'1400003600', u'2146636800'
            )
        url = self.proxy.generate_temp_url(
            self.url,
            2146636800,
            self.method,
            absolute=True,
            temp_url_key=self.key,
        )
        self.assertEqual(url, expected_url)

    def test_generate_temp_url_bad_time(self):
        for bad_time in [
            'not_an_int',
            -1,
            1.1,
            '-1',
            '1.1',
            '2015-05',
            '2015-05-01T01:00',
        ]:
            self.assertRaisesRegex(
                ValueError,
                self.time_errmsg,
                self.proxy.generate_temp_url,
                self.url,
                bad_time,
                self.method,
                temp_url_key=self.key,
            )

    def test_generate_temp_url_bad_path(self):
        for bad_path in [
            '/v1/a/c',
            'v1/a/c/o',
            'blah/v1/a/c/o',
            '/v1//c/o',
            '/v1/a/c/',
            '/v1/a/c',
        ]:
            self.assertRaisesRegex(
                ValueError,
                self.path_errmsg,
                self.proxy.generate_temp_url,
                bad_path,
                60,
                self.method,
                temp_url_key=self.key,
            )


class TestTempURLUnicodePathAndKey(TestTempURL):
    url = u'/v1/\u00e4/c/\u00f3'
    key = u'k\u00e9y'
    expected_url = (
        u'%s?temp_url_sig=temp_url_signature' u'&temp_url_expires=1400003600'
    ) % url
    expected_body = u'\n'.join(
        [
            u'GET',
            u'1400003600',
            url,
        ]
    ).encode('utf-8')


class TestTempURLUnicodePathBytesKey(TestTempURL):
    url = u'/v1/\u00e4/c/\u00f3'
    key = u'k\u00e9y'.encode('utf-8')
    expected_url = (
        u'%s?temp_url_sig=temp_url_signature' u'&temp_url_expires=1400003600'
    ) % url
    expected_body = '\n'.join(
        [
            u'GET',
            u'1400003600',
            url,
        ]
    ).encode('utf-8')


class TestTempURLBytesPathUnicodeKey(TestTempURL):
    url = u'/v1/\u00e4/c/\u00f3'.encode('utf-8')
    key = u'k\u00e9y'
    expected_url = url + (
        b'?temp_url_sig=temp_url_signature' b'&temp_url_expires=1400003600'
    )
    expected_body = b'\n'.join(
        [
            b'GET',
            b'1400003600',
            url,
        ]
    )


class TestTempURLBytesPathAndKey(TestTempURL):
    url = u'/v1/\u00e4/c/\u00f3'.encode('utf-8')
    key = u'k\u00e9y'.encode('utf-8')
    expected_url = url + (
        b'?temp_url_sig=temp_url_signature' b'&temp_url_expires=1400003600'
    )
    expected_body = b'\n'.join(
        [
            b'GET',
            b'1400003600',
            url,
        ]
    )


class TestTempURLBytesPathAndNonUtf8Key(TestTempURL):
    url = u'/v1/\u00e4/c/\u00f3'.encode('utf-8')
    key = b'k\xffy'
    expected_url = url + (
        b'?temp_url_sig=temp_url_signature' b'&temp_url_expires=1400003600'
    )
    expected_body = b'\n'.join(
        [
            b'GET',
            b'1400003600',
            url,
        ]
    )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Connect to an OpenStack cloud.

For a full guide see
https://docs.openstack.org/openstacksdk/latest/user/guides/connect_from_config.html
"""

import argparse
import os
import sys

import openstack
from openstack.config import loader

openstack.enable_logging(True, stream=sys.stdout)

#: Defines the OpenStack Config cloud key in your config file,
#: typically in $HOME/.config/openstack/clouds.yaml. That configuration
#: will determine where the examples will be run and what resource defaults
#: will be used to run the examples.
TEST_CLOUD = os.getenv('OS_TEST_CLOUD', 'devstack-admin')
EXAMPLE_CONFIG_KEY = os.getenv('OPENSTACKSDK_EXAMPLE_CONFIG_KEY', 'example')
config = loader.OpenStackConfig()
cloud = openstack.connect(cloud=TEST_CLOUD)


class Opts:
    def __init__(self, cloud_name='devstack-admin', debug=False):
        self.cloud = cloud_name
        self.debug = debug
        # Use identity v3 API for examples.
        self.identity_api_version = '3'


def _get_resource_value(resource_key, default):
    return config.get_extra_config(EXAMPLE_CONFIG_KEY).get(
        resource_key, default
    )


SERVER_NAME = 'openstacksdk-example'
IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.4.0-x86_64-disk')
FLAVOR_NAME = _get_resource_value('flavor_name', 'm1.small')
NETWORK_NAME = _get_resource_value('network_name', 'private')
KEYPAIR_NAME = _get_resource_value('keypair_name', 'openstacksdk-example')
SSH_DIR = _get_resource_value(
    'ssh_dir', '{home}/.ssh'.format(home=os.path.expanduser("~"))
)
PRIVATE_KEYPAIR_FILE = _get_resource_value(
    'private_keypair_file',
    '{ssh_dir}/id_rsa.{key}'.format(ssh_dir=SSH_DIR, key=KEYPAIR_NAME),
)

EXAMPLE_IMAGE_NAME = 'openstacksdk-example-public-image'


def create_connection_from_config():
    return openstack.connect(cloud=TEST_CLOUD)


def create_connection_from_args():
    parser = argparse.ArgumentParser()
    return openstack.connect(options=parser)


def create_connection(
    auth_url,
    region,
    project_name,
    username,
    password,
    user_domain,
    project_domain,
):
    return openstack.connect(
        auth_url=auth_url,
        project_name=project_name,
        username=username,
        password=password,
        region_name=region,
        user_domain_name=user_domain,
        project_domain_name=project_domain,
        app_name='examples',
        app_version='1.0',
    )



# Copyright 2019 Rackspace, US Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.load_balancer.v2 import provider
from openstack.tests.unit import base


EXAMPLE = {'name': 'best', 'description': 'The best provider'}


class TestProvider(base.TestCase):
    def test_basic(self):
        test_provider = provider.Provider()
        self.assertEqual('providers', test_provider.resources_key)
        self.assertEqual('/lbaas/providers', test_provider.base_path)
        self.assertFalse(test_provider.allow_create)
        self.assertFalse(test_provider.allow_fetch)
        self.assertFalse(test_provider.allow_commit)
        self.assertFalse(test_provider.allow_delete)
        self.assertTrue(test_provider.allow_list)

    def test_make_it(self):
        test_provider = provider.Provider(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], test_provider.name)
        self.assertEqual(EXAMPLE['description'], test_provider.description)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
                'name': 'name',
                'description': 'description',
            },
            test_provider._query_mapping._mapping,
        )


class TestProviderFlavorCapabilities(base.TestCase):
    def test_basic(self):
        test_flav_cap = provider.ProviderFlavorCapabilities()
        self.assertEqual('flavor_capabilities', test_flav_cap.resources_key)
        self.assertEqual(
            '/lbaas/providers/%(provider)s/flavor_capabilities',
            test_flav_cap.base_path,
        )
        self.assertFalse(test_flav_cap.allow_create)
        self.assertFalse(test_flav_cap.allow_fetch)
        self.assertFalse(test_flav_cap.allow_commit)
        self.assertFalse(test_flav_cap.allow_delete)
        self.assertTrue(test_flav_cap.allow_list)

    def test_make_it(self):
        test_flav_cap = provider.ProviderFlavorCapabilities(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], test_flav_cap.name)
        self.assertEqual(EXAMPLE['description'], test_flav_cap.description)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
                'name': 'name',
                'description': 'description',
            },
            test_flav_cap._query_mapping._mapping,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.identity.v3 import role_assignment
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'links': {'self': 'http://example.com/user1'},
    'scope': {'domain': {'id': '2'}},
    'user': {'id': '3'},
    'group': {'id': '4'},
}


class TestRoleAssignment(base.TestCase):
    def test_basic(self):
        sot = role_assignment.RoleAssignment()
        self.assertEqual('role_assignment', sot.resource_key)
        self.assertEqual('role_assignments', sot.resources_key)
        self.assertEqual('/role_assignments', sot.base_path)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = role_assignment.RoleAssignment(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['links'], sot.links)
        self.assertEqual(EXAMPLE['scope'], sot.scope)
        self.assertEqual(EXAMPLE['user'], sot.user)
        self.assertEqual(EXAMPLE['group'], sot.group)



# Copyright (c) 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import openstack.config

ran = []
for cloud in openstack.config.OpenStackConfig().get_all_clouds():
    if cloud.name in ran:
        continue
    ran.append(cloud.name)
    c = cloud.get_session_client('compute')
    try:
        raw_endpoint = c.get_endpoint()
        have_current = False
        endpoint = raw_endpoint.rsplit('/', 2)[0]
        print(endpoint)
        r = c.get(endpoint).json()
    except Exception:
        print("Error with %s" % cloud.name)
        continue
    for version in r['versions']:
        if version['status'] == 'CURRENT':
            have_current = True
            print(
                "\tVersion ID: {id} updated {updated}".format(
                    id=version.get('id'), updated=version.get('updated')
                )
            )
            print("\tVersion Max: {max}".format(max=version.get('version')))
            print(
                "\tVersion Min: {min}".format(min=version.get('min_version'))
            )
    if not have_current:
        for version in r['versions']:
            if version['status'] == 'SUPPORTED':
                have_current = True
                print(
                    "\tVersion ID: {id} updated {updated}".format(
                        id=version.get('id'), updated=version.get('updated')
                    )
                )
                print(
                    "\tVersion Max: {max}".format(max=version.get('version'))
                )
                print(
                    "\tVersion Min: {min}".format(
                        min=version.get('min_version')
                    )
                )



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.network.v2 import firewall_policy
from openstack.network.v2 import firewall_rule
from openstack.tests.functional import base


class TestFirewallPolicyRuleAssociations(base.BaseFunctionalTest):
    POLICY_NAME = uuid.uuid4().hex
    RULE1_NAME = uuid.uuid4().hex
    RULE2_NAME = uuid.uuid4().hex
    POLICY_ID = None
    RULE1_ID = None
    RULE2_ID = None

    def setUp(self):
        super(TestFirewallPolicyRuleAssociations, self).setUp()
        if not self.user_cloud._has_neutron_extension("fwaas_v2"):
            self.skipTest("fwaas_v2 service not supported by cloud")
        rul1 = self.user_cloud.network.create_firewall_rule(
            name=self.RULE1_NAME
        )
        assert isinstance(rul1, firewall_rule.FirewallRule)
        self.assertEqual(self.RULE1_NAME, rul1.name)
        rul2 = self.user_cloud.network.create_firewall_rule(
            name=self.RULE2_NAME
        )
        assert isinstance(rul2, firewall_rule.FirewallRule)
        self.assertEqual(self.RULE2_NAME, rul2.name)
        pol = self.user_cloud.network.create_firewall_policy(
            name=self.POLICY_NAME
        )
        assert isinstance(pol, firewall_policy.FirewallPolicy)
        self.assertEqual(self.POLICY_NAME, pol.name)
        self.RULE1_ID = rul1.id
        self.RULE2_ID = rul2.id
        self.POLICY_ID = pol.id

    def tearDown(self):
        sot = self.user_cloud.network.delete_firewall_policy(
            self.POLICY_ID, ignore_missing=False
        )
        self.assertIs(None, sot)
        sot = self.user_cloud.network.delete_firewall_rule(
            self.RULE1_ID, ignore_missing=False
        )
        self.assertIs(None, sot)
        sot = self.user_cloud.network.delete_firewall_rule(
            self.RULE2_ID, ignore_missing=False
        )
        self.assertIs(None, sot)
        super(TestFirewallPolicyRuleAssociations, self).tearDown()

    def test_insert_rule_into_policy(self):
        policy = self.user_cloud.network.insert_rule_into_policy(
            self.POLICY_ID, firewall_rule_id=self.RULE1_ID
        )
        self.assertIn(self.RULE1_ID, policy["firewall_rules"])
        policy = self.user_cloud.network.insert_rule_into_policy(
            self.POLICY_ID,
            firewall_rule_id=self.RULE2_ID,
            insert_before=self.RULE1_ID,
        )
        self.assertEqual(self.RULE1_ID, policy["firewall_rules"][1])
        self.assertEqual(self.RULE2_ID, policy["firewall_rules"][0])

    def test_remove_rule_from_policy(self):
        # insert rules into policy before we remove it again
        policy = self.user_cloud.network.insert_rule_into_policy(
            self.POLICY_ID, firewall_rule_id=self.RULE1_ID
        )
        self.assertIn(self.RULE1_ID, policy["firewall_rules"])

        policy = self.user_cloud.network.insert_rule_into_policy(
            self.POLICY_ID, firewall_rule_id=self.RULE2_ID
        )
        self.assertIn(self.RULE2_ID, policy["firewall_rules"])

        policy = self.user_cloud.network.remove_rule_from_policy(
            self.POLICY_ID, firewall_rule_id=self.RULE1_ID
        )
        self.assertNotIn(self.RULE1_ID, policy["firewall_rules"])

        policy = self.user_cloud.network.remove_rule_from_policy(
            self.POLICY_ID, firewall_rule_id=self.RULE2_ID
        )
        self.assertNotIn(self.RULE2_ID, policy["firewall_rules"])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import type
from openstack.tests.unit import base


FAKE_ID = "479394ab-2f25-416e-8f58-721d8e5e29de"
TYPE_ID = "22373aed-c4a8-4072-b66c-bf0a90dc9a12"
TYPE_ENC = {
    "key_size": 256,
    "volume_type_id": TYPE_ID,
    "encryption_id": FAKE_ID,
    "provider": "nova.volume.encryptors.luks.LuksEncryptor",
    "control_location": "front-end",
    "cipher": "aes-xts-plain64",
    "deleted": False,
    "created_at": "2020-10-07T07:52:30.000000",
    "updated_at": "2020-10-08T07:42:45.000000",
    "deleted_at": None,
}


class TestTypeEncryption(base.TestCase):
    def test_basic(self):
        sot = type.TypeEncryption(**TYPE_ENC)
        self.assertEqual("encryption", sot.resource_key)
        self.assertEqual("encryption", sot.resources_key)
        self.assertEqual("/types/%(volume_type_id)s/encryption", sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertTrue(sot.allow_commit)

    def test_new(self):
        sot = type.TypeEncryption.new(encryption_id=FAKE_ID)
        self.assertEqual(FAKE_ID, sot.encryption_id)

    def test_create(self):
        sot = type.TypeEncryption(**TYPE_ENC)
        self.assertEqual(TYPE_ENC["volume_type_id"], sot.volume_type_id)
        self.assertEqual(TYPE_ENC["encryption_id"], sot.encryption_id)
        self.assertEqual(TYPE_ENC["key_size"], sot.key_size)
        self.assertEqual(TYPE_ENC["provider"], sot.provider)
        self.assertEqual(TYPE_ENC["control_location"], sot.control_location)
        self.assertEqual(TYPE_ENC["cipher"], sot.cipher)
        self.assertEqual(TYPE_ENC["deleted"], sot.deleted)
        self.assertEqual(TYPE_ENC["created_at"], sot.created_at)
        self.assertEqual(TYPE_ENC["updated_at"], sot.updated_at)
        self.assertEqual(TYPE_ENC["deleted_at"], sot.deleted_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
List resources from the Shared File System service.

For a full guide see
https://docs.openstack.org/openstacksdk/latest/user/guides/shared_file_system.html
"""


def list_share_group_snapshots(conn, **query):
    print("List all share group snapshots:")
    share_group_snapshots = conn.share.share_group_snapshots(**query)
    for share_group_snapshot in share_group_snapshots:
        print(share_group_snapshot)


def get_share_group_snapshot(conn, group_snapshot_id):
    print("Show share group snapshot with given Id:")
    share_group_snapshot = conn.share.get_share_group_snapshots(
        group_snapshot_id
    )
    print(share_group_snapshot)


def share_group_snapshot_members(conn, group_snapshot_id):
    print("Show share group snapshot members with given Id:")
    members = conn.share.share_group_snapshot_members(group_snapshot_id)
    for member in members:
        print(member)


def create_share_group_snapshot(conn, share_group_id, **attrs):
    print("Creating a share group snapshot from given attributes:")
    share_group_snapshot = conn.share.create_share_group_snapshot(
        share_group_id, **attrs
    )
    print(share_group_snapshot)


def reset_share_group_snapshot_status(conn, group_snapshot_id, status):
    print("Reseting the share group snapshot status:")
    conn.share.reset_share_group_snapshot_status(group_snapshot_id, status)


def update_share_group_snapshot(conn, group_snapshot_id, **attrs):
    print("Updating a share group snapshot with given Id:")
    share_group_snapshot = conn.share.update_share_group_snapshot(
        group_snapshot_id, **attrs
    )
    print(share_group_snapshot)


def delete_share_group_snapshot(conn, group_snapshot_id):
    print("Deleting a share group snapshot with given Id:")
    conn.share.delete_share_group_snapshot(group_snapshot_id)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools
from testtools import matchers

from openstack import exceptions
from openstack.tests.unit import base


RAW_ROLE_ASSIGNMENTS = [
    {
        "links": {"assignment": "http://example"},
        "role": {"id": "123456"},
        "scope": {"domain": {"id": "161718"}},
        "user": {"id": "313233"},
    },
    {
        "links": {"assignment": "http://example"},
        "group": {"id": "101112"},
        "role": {"id": "123456"},
        "scope": {"project": {"id": "456789"}},
    },
]


class TestIdentityRoles(base.TestCase):
    def get_mock_url(
        self,
        service_type='identity',
        interface='public',
        resource='roles',
        append=None,
        base_url_append='v3',
        qs_elements=None,
    ):
        return super(TestIdentityRoles, self).get_mock_url(
            service_type,
            interface,
            resource,
            append,
            base_url_append,
            qs_elements,
        )

    def test_list_roles(self):
        role_data = self._get_role_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={'roles': [role_data.json_response['role']]},
                )
            ]
        )
        self.cloud.list_roles()
        self.assert_calls()

    def test_list_role_by_name(self):
        role_data = self._get_role_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        qs_elements=['name={0}'.format(role_data.role_name)]
                    ),
                    status_code=200,
                    json={'roles': [role_data.json_response['role']]},
                )
            ]
        )
        role = self.cloud.list_roles(name=role_data.role_name)[0]

        self.assertIsNotNone(role)
        self.assertThat(role.id, matchers.Equals(role_data.role_id))
        self.assertThat(role.name, matchers.Equals(role_data.role_name))
        self.assert_calls()

    def test_get_role_by_name(self):
        role_data = self._get_role_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={'roles': [role_data.json_response['role']]},
                )
            ]
        )
        role = self.cloud.get_role(role_data.role_name)

        self.assertIsNotNone(role)
        self.assertThat(role.id, matchers.Equals(role_data.role_id))
        self.assertThat(role.name, matchers.Equals(role_data.role_name))
        self.assert_calls()

    def test_get_role_by_id(self):
        role_data = self._get_role_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={'roles': [role_data.json_response['role']]},
                )
            ]
        )
        role = self.cloud.get_role(role_data.role_id)

        self.assertIsNotNone(role)
        self.assertThat(role.id, matchers.Equals(role_data.role_id))
        self.assertThat(role.name, matchers.Equals(role_data.role_name))
        self.assert_calls()

    def test_create_role(self):
        role_data = self._get_role_data()
        self.register_uris(
            [
                dict(
                    method='POST',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json=role_data.json_response,
                    validate=dict(json=role_data.json_request),
                )
            ]
        )

        role = self.cloud.create_role(role_data.role_name)

        self.assertIsNotNone(role)
        self.assertThat(role.name, matchers.Equals(role_data.role_name))
        self.assertThat(role.id, matchers.Equals(role_data.role_id))
        self.assert_calls()

    def test_update_role(self):
        role_data = self._get_role_data()
        req = {'role': {'name': 'new_name'}}
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={'roles': [role_data.json_response['role']]},
                ),
                dict(
                    method='PATCH',
                    uri=self.get_mock_url(append=[role_data.role_id]),
                    status_code=200,
                    json=role_data.json_response,
                    validate=dict(json=req),
                ),
            ]
        )

        role = self.cloud.update_role(role_data.role_id, 'new_name')

        self.assertIsNotNone(role)
        self.assertThat(role.name, matchers.Equals(role_data.role_name))
        self.assertThat(role.id, matchers.Equals(role_data.role_id))
        self.assert_calls()

    def test_delete_role_by_id(self):
        role_data = self._get_role_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={'roles': [role_data.json_response['role']]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(append=[role_data.role_id]),
                    status_code=204,
                ),
            ]
        )
        role = self.cloud.delete_role(role_data.role_id)
        self.assertThat(role, matchers.Equals(True))
        self.assert_calls()

    def test_delete_role_by_name(self):
        role_data = self._get_role_data()
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(),
                    status_code=200,
                    json={'roles': [role_data.json_response['role']]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(append=[role_data.role_id]),
                    status_code=204,
                ),
            ]
        )
        role = self.cloud.delete_role(role_data.role_name)
        self.assertThat(role, matchers.Equals(True))
        self.assert_calls()

    def test_list_role_assignments(self):
        domain_data = self._get_domain_data()
        user_data = self._get_user_data(domain_id=domain_data.domain_id)
        group_data = self._get_group_data(domain_id=domain_data.domain_id)
        project_data = self._get_project_data(domain_id=domain_data.domain_id)
        role_data = self._get_role_data()
        response = [
            {
                'links': 'https://example.com',
                'role': {'id': role_data.role_id},
                'scope': {'domain': {'id': domain_data.domain_id}},
                'user': {'id': user_data.user_id},
            },
            {
                'links': 'https://example.com',
                'role': {'id': role_data.role_id},
                'scope': {'project': {'id': project_data.project_id}},
                'group': {'id': group_data.group_id},
            },
        ]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(resource='role_assignments'),
                    status_code=200,
                    json={'role_assignments': response},
                    complete_qs=True,
                )
            ]
        )
        ret = self.cloud.list_role_assignments()
        self.assertThat(len(ret), matchers.Equals(2))
        self.assertThat(ret[0].user['id'], matchers.Equals(user_data.user_id))
        self.assertThat(ret[0].role['id'], matchers.Equals(role_data.role_id))
        self.assertThat(
            ret[0].scope['domain']['id'],
            matchers.Equals(domain_data.domain_id),
        )
        self.assertThat(
            ret[1].group['id'], matchers.Equals(group_data.group_id)
        )
        self.assertThat(ret[1].role['id'], matchers.Equals(role_data.role_id))
        self.assertThat(
            ret[1].scope['project']['id'],
            matchers.Equals(project_data.project_id),
        )

    def test_list_role_assignments_filters(self):
        domain_data = self._get_domain_data()
        user_data = self._get_user_data(domain_id=domain_data.domain_id)
        role_data = self._get_role_data()
        response = [
            {
                'links': 'https://example.com',
                'role': {'id': role_data.role_id},
                'scope': {'domain': {'id': domain_data.domain_id}},
                'user': {'id': user_data.user_id},
            }
        ]

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        resource='role_assignments',
                        qs_elements=[
                            'scope.domain.id=%s' % domain_data.domain_id,
                            'user.id=%s' % user_data.user_id,
                            'effective=True',
                        ],
                    ),
                    status_code=200,
                    json={'role_assignments': response},
                    complete_qs=True,
                )
            ]
        )
        params = dict(
            user=user_data.user_id,
            domain=domain_data.domain_id,
            effective=True,
        )
        ret = self.cloud.list_role_assignments(filters=params)
        self.assertThat(len(ret), matchers.Equals(1))
        self.assertThat(ret[0].user['id'], matchers.Equals(user_data.user_id))
        self.assertThat(ret[0].role['id'], matchers.Equals(role_data.role_id))
        self.assertThat(
            ret[0].scope['domain']['id'],
            matchers.Equals(domain_data.domain_id),
        )

    def test_list_role_assignments_exception(self):
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(resource='role_assignments'),
                    status_code=403,
                )
            ]
        )
        with testtools.ExpectedException(exceptions.ForbiddenException):
            self.cloud.list_role_assignments()
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v2 import member
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'created_at': '2015-03-09T12:14:57.233772',
    'image_id': '2',
    'member_id': IDENTIFIER,
    'status': '4',
    'updated_at': '2015-03-09T12:15:57.233772',
}


class TestMember(base.TestCase):
    def test_basic(self):
        sot = member.Member()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('members', sot.resources_key)
        self.assertEqual('/images/%(image_id)s/members', sot.base_path)
        self.assertEqual('member', sot._alternate_id())
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = member.Member(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['image_id'], sot.image_id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.block_storage.v3 import backup
from openstack import exceptions
from openstack.tests.unit import base


FAKE_ID = "6685584b-1eac-4da6-b5c3-555430cf68ff"

BACKUP = {
    "availability_zone": "az1",
    "container": "volumebackups",
    "created_at": "2018-04-02T10:35:27.000000",
    "updated_at": "2018-04-03T10:35:27.000000",
    "description": 'description',
    "encryption_key_id": "fake_encry_id",
    "fail_reason": 'fail reason',
    "id": FAKE_ID,
    "name": "backup001",
    "object_count": 22,
    "size": 1,
    "status": "available",
    "volume_id": "e5185058-943a-4cb4-96d9-72c184c337d6",
    "is_incremental": True,
    "has_dependent_backups": False,
    "os-backup-project-attr:project_id": "2c67a14be9314c5dae2ee6c4ec90cf0b",
    "user_id": "515ba0dd59f84f25a6a084a45d8d93b2",
    "metadata": {"key": "value"},
}


class TestBackup(base.TestCase):
    def setUp(self):
        super(TestBackup, self).setUp()
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.resp.headers = {}
        self.resp.status_code = 202

        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock(return_value=self.resp)
        self.sess.default_microversion = "3.64"

    def test_basic(self):
        sot = backup.Backup(BACKUP)
        self.assertEqual("backup", sot.resource_key)
        self.assertEqual("backups", sot.resources_key)
        self.assertEqual("/backups", sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_fetch)
        self.assertIsNotNone(sot._max_microversion)

        self.assertDictEqual(
            {
                "limit": "limit",
                "marker": "marker",
                "offset": "offset",
                "project_id": "project_id",
                "name": "name",
                "status": "status",
                "volume_id": "volume_id",
                "sort_dir": "sort_dir",
                "sort_key": "sort_key",
                "sort": "sort",
                "all_projects": "all_tenants",
            },
            sot._query_mapping._mapping,
        )

    def test_create(self):
        sot = backup.Backup(**BACKUP)
        self.assertEqual(BACKUP["id"], sot.id)
        self.assertEqual(BACKUP["name"], sot.name)
        self.assertEqual(BACKUP["status"], sot.status)
        self.assertEqual(BACKUP["container"], sot.container)
        self.assertEqual(BACKUP["availability_zone"], sot.availability_zone)
        self.assertEqual(BACKUP["created_at"], sot.created_at)
        self.assertEqual(BACKUP["updated_at"], sot.updated_at)
        self.assertEqual(BACKUP["description"], sot.description)
        self.assertEqual(BACKUP["fail_reason"], sot.fail_reason)
        self.assertEqual(BACKUP["volume_id"], sot.volume_id)
        self.assertEqual(BACKUP["object_count"], sot.object_count)
        self.assertEqual(BACKUP["is_incremental"], sot.is_incremental)
        self.assertEqual(BACKUP["size"], sot.size)
        self.assertEqual(
            BACKUP["has_dependent_backups"], sot.has_dependent_backups
        )
        self.assertEqual(
            BACKUP['os-backup-project-attr:project_id'], sot.project_id
        )
        self.assertEqual(BACKUP['metadata'], sot.metadata)
        self.assertEqual(BACKUP['user_id'], sot.user_id)
        self.assertEqual(BACKUP['encryption_key_id'], sot.encryption_key_id)

    def test_create_incremental(self):
        sot = backup.Backup(is_incremental=True)
        sot2 = backup.Backup(is_incremental=False)

        create_response = mock.Mock()
        create_response.status_code = 200
        create_response.json.return_value = {}
        create_response.headers = {}
        self.sess.post.return_value = create_response

        sot.create(self.sess)
        self.sess.post.assert_called_with(
            '/backups',
            headers={},
            json={
                'backup': {
                    'incremental': True,
                }
            },
            microversion="3.64",
            params={},
        )

        sot2.create(self.sess)
        self.sess.post.assert_called_with(
            '/backups',
            headers={},
            json={
                'backup': {
                    'incremental': False,
                }
            },
            microversion="3.64",
            params={},
        )

    def test_restore(self):
        sot = backup.Backup(**BACKUP)

        self.assertEqual(sot, sot.restore(self.sess, 'vol', 'name'))

        url = 'backups/%s/restore' % FAKE_ID
        body = {"restore": {"volume_id": "vol", "name": "name"}}
        self.sess.post.assert_called_with(url, json=body)

    def test_restore_name(self):
        sot = backup.Backup(**BACKUP)

        self.assertEqual(sot, sot.restore(self.sess, name='name'))

        url = 'backups/%s/restore' % FAKE_ID
        body = {"restore": {"name": "name"}}
        self.sess.post.assert_called_with(url, json=body)

    def test_restore_vol_id(self):
        sot = backup.Backup(**BACKUP)

        self.assertEqual(sot, sot.restore(self.sess, volume_id='vol'))

        url = 'backups/%s/restore' % FAKE_ID
        body = {"restore": {"volume_id": "vol"}}
        self.sess.post.assert_called_with(url, json=body)

    def test_restore_no_params(self):
        sot = backup.Backup(**BACKUP)

        self.assertRaises(exceptions.SDKException, sot.restore, self.sess)

    def test_force_delete(self):
        sot = backup.Backup(**BACKUP)

        self.assertIsNone(sot.force_delete(self.sess))

        url = 'backups/%s/action' % FAKE_ID
        body = {'os-force_delete': None}
        self.sess.post.assert_called_with(
            url, json=body, microversion=sot._max_microversion
        )

    def test_reset(self):
        sot = backup.Backup(**BACKUP)

        self.assertIsNone(sot.reset(self.sess, 'new_status'))

        url = 'backups/%s/action' % FAKE_ID
        body = {'os-reset_status': {'status': 'new_status'}}
        self.sess.post.assert_called_with(
            url, json=body, microversion=sot._max_microversion
        )



# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io
import logging
import os
import pprint
import sys
import typing as ty

import fixtures
from oslotest import base
import testtools.content

from openstack.tests import fixtures as os_fixtures
from openstack import utils

_TRUE_VALUES = ('true', '1', 'yes')


class TestCase(base.BaseTestCase):

    """Test case base class for all tests."""

    # A way to adjust slow test classes
    TIMEOUT_SCALING_FACTOR = 1.0

    def setUp(self):
        """Run before each test method to initialize test environment."""
        # No openstacksdk unit tests should EVER run longer than a second.
        # Set this to 5 by default just to give us some fudge.
        # Do this before super setUp so that we intercept the default value
        # in oslotest. TODO(mordred) Make the default timeout configurable
        # in oslotest.
        test_timeout = int(os.environ.get('OS_TEST_TIMEOUT', '5'))
        try:
            test_timeout = int(test_timeout * self.TIMEOUT_SCALING_FACTOR)
            self.useFixture(
                fixtures.EnvironmentVariable(
                    'OS_TEST_TIMEOUT', str(test_timeout)
                )
            )
        except ValueError:
            # Let oslotest do its thing
            pass

        super().setUp()

        self.warnings = self.useFixture(os_fixtures.WarningsFixture())

        self._log_stream: ty.TextIO

        if os.environ.get('OS_LOG_CAPTURE') in _TRUE_VALUES:
            self._log_stream = io.StringIO()
            if os.environ.get('OS_ALWAYS_LOG') in _TRUE_VALUES:
                self.addCleanup(self.printLogs)
            else:
                self.addOnException(self.attachLogs)
        else:
            self._log_stream = sys.stdout

        handler = logging.StreamHandler(self._log_stream)
        formatter = logging.Formatter('%(asctime)s %(name)-32s %(message)s')
        handler.setFormatter(formatter)

        logger = logging.getLogger('openstack')
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)

        # Enable HTTP level tracing
        # TODO(mordred) This is blowing out our memory we think
        logger = logging.getLogger('keystoneauth')
        logger.setLevel(logging.INFO)
        logger.addHandler(handler)
        logger.propagate = False

    def _fake_logs(self):
        # Override _fake_logs in oslotest until we can get our
        # attach-on-exception logic added
        pass

    def assertEqual(self, first, second, *args, **kwargs):
        '''Munch aware wrapper'''
        if isinstance(first, utils.Munch):
            first = first.toDict()
        if isinstance(second, utils.Munch):
            second = second.toDict()
        return super(TestCase, self).assertEqual(
            first, second, *args, **kwargs
        )

    def printLogs(self, *args):
        self._log_stream.seek(0)
        print(self._log_stream.read())

    def attachLogs(self, *args):
        def reader():
            self._log_stream.seek(0)
            while True:
                x = self._log_stream.read(4096)
                if not x:
                    break
                yield x.encode('utf8')

        content = testtools.content.content_from_reader(
            reader, testtools.content_type.UTF8_TEXT, False
        )
        self.addDetail('logging', content)

    def add_info_on_exception(self, name, text):
        def add_content(unused):
            self.addDetail(
                name, testtools.content.text_content(pprint.pformat(text))
            )

        self.addOnException(add_content)

    def assertSubdict(self, part, whole):
        missing_keys = []
        for key in part:
            # In the resource we have virtual access by not existing keys. To
            # verify those are there try access it.
            if not whole[key] and part[key]:
                missing_keys.append(key)
        if missing_keys:
            self.fail(
                "Keys %s are in %s but not in %s" % (missing_keys, part, whole)
            )
        wrong_values = [
            (key, part[key], whole[key])
            for key in part
            if part[key] != whole[key]
        ]
        if wrong_values:
            self.fail(
                "Mismatched values: %s"
                % ", ".join(
                    "for %s got %s and %s" % tpl for tpl in wrong_values
                )
            )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ShareNetworkSubnet(resource.Resource):
    resource_key = "share_network_subnet"
    resources_key = "share_network_subnets"
    base_path = "/share-networks/%(share_network_id)s/subnets"

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = False
    allow_delete = True
    allow_list = True

    #: Properties
    #: The share nerwork ID, part of the URI for share network subnets.
    share_network_id = resource.URI("share_network_id", type=str)

    #: The name of the availability zone that the share network
    #: subnet belongs to.
    availability_zone = resource.Body("availability_zone", type=str)
    #: The IP block from which to allocate the network, in CIDR notation.
    cidr = resource.Body("cidr", type=str)
    #: Date and time the share network subnet was created at.
    created_at = resource.Body("created_at")
    #: The gateway of a share network subnet.
    gateway = resource.Body("gateway", type=str)
    #: The IP version of the network.
    ip_version = resource.Body("ip_version", type=int)
    #: The MTU of a share network subnet.
    mtu = resource.Body("mtu", type=str)
    #: The network type. A valid value is VLAN, VXLAN, GRE, or flat
    network_type = resource.Body("network_type", type=str)
    #: The name of the neutron network.
    neutron_net_id = resource.Body("neutron_net_id", type=str)
    #: The ID of the neitron subnet.
    neutron_subnet_id = resource.Body("neutron_subnet_id", type=str)
    #: The segmentation ID.
    segmentation_id = resource.Body('segmentation_id', type=int)
    #: The name of the share network that the share network subnet belongs to.
    share_network_name = resource.Body("share_network_name", type=str)
    #: Date and time the share network subnet was last updated at.
    updated_at = resource.Body("updated_at", type=str)

    def create(self, session, **kwargs):
        return super().create(
            session, resource_request_key='share-network-subnet', **kwargs
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ApplicationCredential(resource.Resource):
    resource_key = 'application_credential'
    resources_key = 'application_credentials'
    base_path = '/users/%(user_id)s/application_credentials'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # Properties
    #: User ID using application credential. *Type: string*
    user_id = resource.URI('user_id')
    #: User object using application credential. *Type: string*
    user = resource.Body('user')
    #: The links for the application credential resource.
    links = resource.Body('links')
    #: name of the user. *Type: string*
    name = resource.Body('name')
    #: secret that application credential will be created with, if any.
    # *Type: string*
    secret = resource.Body('secret')
    #: description of application credential's purpose. *Type: string*
    description = resource.Body('description')
    #: expire time of application credential. *Type: string*
    expires_at = resource.Body('expires_at')
    #: roles of the user. *Type: list*
    roles = resource.Body('roles')
    #: restricts the application credential. *Type: boolean*
    unrestricted = resource.Body('unrestricted', type=bool)
    #: ID of project. *Type: string*
    project_id = resource.Body('project_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional.block_storage.v3 import base
from openstack import utils


class TestTransfer(base.BaseBlockStorageTest):
    def setUp(self):
        super().setUp()

        self.VOLUME_NAME = self.getUniqueString()

        self.volume = self.user_cloud.block_storage.create_volume(
            name=self.VOLUME_NAME,
            size=1,
        )
        self.user_cloud.block_storage.wait_for_status(
            self.volume,
            status='available',
            failures=['error'],
            interval=2,
            wait=self._wait_for_timeout,
        )
        self.VOLUME_ID = self.volume.id

    def tearDown(self):
        sot = self.user_cloud.block_storage.delete_volume(
            self.VOLUME_ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        super().tearDown()

    def test_transfer(self):
        if not utils.supports_microversion(self.conn.block_storage, "3.55"):
            self.skipTest("Cannot test new transfer API if MV < 3.55")
        sot = self.conn.block_storage.create_transfer(
            volume_id=self.VOLUME_ID,
            name=self.VOLUME_NAME,
        )
        self.assertIn('auth_key', sot)
        self.assertIn('created_at', sot)
        self.assertIn('id', sot)
        self.assertIn('name', sot)
        self.assertIn('volume_id', sot)

        sot = self.user_cloud.block_storage.delete_transfer(
            sot.id, ignore_missing=False
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.clustering.v1 import cluster_attr as ca
from openstack.tests.unit import base


FAKE = {
    'cluster_id': '633bd3c6-520b-420f-8e6a-dc2a47022b53',
    'path': 'path.to.attr',
    'id': 'c378e474-d091-43a3-b083-e19719291358',
    'value': 'fake value',
}


class TestClusterAttr(base.TestCase):
    def setUp(self):
        super(TestClusterAttr, self).setUp()

    def test_basic(self):
        sot = ca.ClusterAttr()
        self.assertEqual('cluster_attributes', sot.resources_key)
        self.assertEqual(
            '/clusters/%(cluster_id)s/attrs/%(path)s', sot.base_path
        )
        self.assertTrue(sot.allow_list)

    def test_instantiate(self):
        sot = ca.ClusterAttr(**FAKE)
        self.assertEqual(FAKE['cluster_id'], sot.cluster_id)
        self.assertEqual(FAKE['path'], sot.path)
        self.assertEqual(FAKE['id'], sot.node_id)
        self.assertEqual(FAKE['value'], sot.attr_value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.network.v2 import bgp_speaker
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'name': 'bgp-speaker',
    'peers': [],
    'ip_version': 4,
    'advertise_floating_ip_host_routes': 'true',
    'advertise_tenant_networks': 'true',
    'local_as': 1000,
    'networks': [],
    'project_id': '42',
}


class TestBgpSpeaker(base.TestCase):
    def test_basic(self):
        sot = bgp_speaker.BgpSpeaker()
        self.assertEqual('bgp_speaker', sot.resource_key)
        self.assertEqual('bgp_speakers', sot.resources_key)
        self.assertEqual('/bgp-speakers', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['ip_version'], sot.ip_version)
        self.assertEqual(
            EXAMPLE['advertise_floating_ip_host_routes'],
            sot.advertise_floating_ip_host_routes,
        )
        self.assertEqual(EXAMPLE['local_as'], sot.local_as)
        self.assertEqual(EXAMPLE['networks'], sot.networks)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
            },
            sot._query_mapping._mapping,
        )

    def test_add_bgp_peer(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        response = mock.Mock()
        response.body = {'bgp_peer_id': '101'}
        response.json = mock.Mock(return_value=response.body)
        response.status_code = 200
        sess = mock.Mock()
        sess.put = mock.Mock(return_value=response)
        ret = sot.add_bgp_peer(sess, '101')
        self.assertIsInstance(ret, dict)
        self.assertEqual(ret, {'bgp_peer_id': '101'})

        body = {'bgp_peer_id': '101'}
        url = 'bgp-speakers/IDENTIFIER/add_bgp_peer'
        sess.put.assert_called_with(url, json=body)

    def test_remove_bgp_peer(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        response = mock.Mock()
        response.body = {'bgp_peer_id': '102'}
        response.json = mock.Mock(return_value=response.body)
        response.status_code = 200
        sess = mock.Mock()
        sess.put = mock.Mock(return_value=response)
        ret = sot.remove_bgp_peer(sess, '102')
        self.assertIsNone(ret)

        body = {'bgp_peer_id': '102'}
        url = 'bgp-speakers/IDENTIFIER/remove_bgp_peer'
        sess.put.assert_called_with(url, json=body)

    def test_add_gateway_network(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        response = mock.Mock()
        response.body = {'network_id': 'net_id'}
        response.json = mock.Mock(return_value=response.body)
        response.status_code = 200
        sess = mock.Mock()
        sess.put = mock.Mock(return_value=response)
        ret = sot.add_gateway_network(sess, 'net_id')
        self.assertIsInstance(ret, dict)
        self.assertEqual(ret, {'network_id': 'net_id'})

        body = {'network_id': 'net_id'}
        url = 'bgp-speakers/IDENTIFIER/add_gateway_network'
        sess.put.assert_called_with(url, json=body)

    def test_remove_gateway_network(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        response = mock.Mock()
        response.body = {'network_id': 'net_id42'}
        response.json = mock.Mock(return_value=response.body)
        response.status_code = 200
        sess = mock.Mock()
        sess.put = mock.Mock(return_value=response)
        ret = sot.remove_gateway_network(sess, 'net_id42')
        self.assertIsNone(ret)

        body = {'network_id': 'net_id42'}
        url = 'bgp-speakers/IDENTIFIER/remove_gateway_network'
        sess.put.assert_called_with(url, json=body)

    def test_get_advertised_routes(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        response = mock.Mock()
        response.body = {
            'advertised_routes': [
                {'cidr': '192.168.10.0/24', 'nexthop': '10.0.0.1'}
            ]
        }
        response.json = mock.Mock(return_value=response.body)
        response.status_code = 200
        sess = mock.Mock()
        sess.get = mock.Mock(return_value=response)
        ret = sot.get_advertised_routes(sess)

        url = 'bgp-speakers/IDENTIFIER/get_advertised_routes'
        sess.get.assert_called_with(url)
        self.assertEqual(ret, response.body)

    def test_get_bgp_dragents(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        response = mock.Mock()
        response.body = {
            'agents': [{'binary': 'neutron-bgp-dragent', 'alive': True}]
        }
        response.json = mock.Mock(return_value=response.body)
        response.status_code = 200
        sess = mock.Mock()
        sess.get = mock.Mock(return_value=response)
        ret = sot.get_bgp_dragents(sess)

        url = 'bgp-speakers/IDENTIFIER/bgp-dragents'
        sess.get.assert_called_with(url)
        self.assertEqual(ret, response.body)

    def test_add_bgp_speaker_to_dragent(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        agent_id = '123-42'
        response = mock.Mock()
        response.status_code = 201
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)
        self.assertIsNone(sot.add_bgp_speaker_to_dragent(sess, agent_id))

        body = {'bgp_speaker_id': sot.id}
        url = 'agents/%s/bgp-drinstances' % agent_id
        sess.post.assert_called_with(url, json=body)

    def test_remove_bgp_speaker_from_dragent(self):
        sot = bgp_speaker.BgpSpeaker(**EXAMPLE)
        agent_id = '123-42'
        response = mock.Mock()
        response.status_code = 204
        sess = mock.Mock()
        sess.delete = mock.Mock(return_value=response)
        self.assertIsNone(sot.remove_bgp_speaker_from_dragent(sess, agent_id))

        url = 'agents/%s/bgp-drinstances/%s' % (agent_id, IDENTIFIER)
        sess.delete.assert_called_with(url)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import security_group
from openstack.tests.functional import base


class TestSecurityGroup(base.BaseFunctionalTest):
    ID = None

    def setUp(self):
        super(TestSecurityGroup, self).setUp()
        self.NAME = self.getUniqueString()
        sot = self.user_cloud.network.create_security_group(name=self.NAME)
        assert isinstance(sot, security_group.SecurityGroup)
        self.assertEqual(self.NAME, sot.name)
        self.ID = sot.id

    def tearDown(self):
        sot = self.user_cloud.network.delete_security_group(
            self.ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        super(TestSecurityGroup, self).tearDown()

    def test_find(self):
        sot = self.user_cloud.network.find_security_group(self.NAME)
        self.assertEqual(self.ID, sot.id)

    def test_get(self):
        sot = self.user_cloud.network.get_security_group(self.ID)
        self.assertEqual(self.NAME, sot.name)
        self.assertEqual(self.ID, sot.id)

    def test_list(self):
        names = [o.name for o in self.user_cloud.network.security_groups()]
        self.assertIn(self.NAME, names)

    def test_list_query_list_of_ids(self):
        ids = [
            o.id for o in self.user_cloud.network.security_groups(id=[self.ID])
        ]
        self.assertIn(self.ID, ids)

    def test_set_tags(self):
        sot = self.user_cloud.network.get_security_group(self.ID)
        self.assertEqual([], sot.tags)

        self.user_cloud.network.set_tags(sot, ["blue"])
        sot = self.user_cloud.network.get_security_group(self.ID)
        self.assertEqual(["blue"], sot.tags)

        self.user_cloud.network.set_tags(sot, [])
        sot = self.user_cloud.network.get_security_group(self.ID)
        self.assertEqual([], sot.tags)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack.cloud import _utils
from openstack.cloud import exc
from openstack import exceptions
from openstack.network.v2._proxy import Proxy
from openstack import proxy
from openstack import utils


class SecurityGroupCloudMixin:
    network: Proxy

    def __init__(self):
        self.secgroup_source = self.config.config['secgroup_source']

    def search_security_groups(self, name_or_id=None, filters=None):
        # `filters` could be a dict or a jmespath (str)
        groups = self.list_security_groups(
            filters=filters if isinstance(filters, dict) else None
        )
        return _utils._filter_list(groups, name_or_id, filters)

    def list_security_groups(self, filters=None):
        """List all available security groups.

        :param filters: (optional) dict of filter conditions to push down
        :returns: A list of security group
            ``openstack.network.v2.security_group.SecurityGroup``.

        """
        # Security groups not supported
        if not self._has_secgroups():
            raise exc.OpenStackCloudUnavailableFeature(
                "Unavailable feature: security groups"
            )

        if not filters:
            filters = {}

        data = []
        # Handle neutron security groups
        if self._use_neutron_secgroups():
            # pass filters dict to the list to filter as much as possible on
            # the server side
            return list(self.network.security_groups(**filters))

        # Handle nova security groups
        else:
            data = proxy._json_response(
                self.compute.get('/os-security-groups', params=filters)
            )
        return self._normalize_secgroups(
            self._get_and_munchify('security_groups', data)
        )

    def get_security_group(self, name_or_id, filters=None):
        """Get a security group by name or ID.

        :param name_or_id: Name or ID of the security group.
        :param filters:
            A dictionary of meta data to use for further filtering. Elements
            of this dictionary may, themselves, be dictionaries. Example::

                {
                    'last_name': 'Smith',
                    'other': {
                        'gender': 'Female'
                    }
                }

            OR
            A string containing a jmespath expression for further filtering.
            Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"

        :returns: A security group
            ``openstack.network.v2.security_group.SecurityGroup``
            or None if no matching security group is found.

        """
        return _utils._get_entity(self, 'security_group', name_or_id, filters)

    def get_security_group_by_id(self, id):
        """Get a security group by ID

        :param id: ID of the security group.
        :returns: A security group
            ``openstack.network.v2.security_group.SecurityGroup``.
        """
        if not self._has_secgroups():
            raise exc.OpenStackCloudUnavailableFeature(
                "Unavailable feature: security groups"
            )
        error_message = f"Error getting security group with ID {id}"
        if self._use_neutron_secgroups():
            return self.network.get_security_group(id)
        else:
            data = proxy._json_response(
                self.compute.get(f'/os-security-groups/{id}'),
                error_message=error_message,
            )
        return self._normalize_secgroup(
            self._get_and_munchify('security_group', data)
        )

    def create_security_group(
        self, name, description, project_id=None, stateful=None
    ):
        """Create a new security group

        :param string name: A name for the security group.
        :param string description: Describes the security group.
        :param string project_id:
            Specify the project ID this security group will be created
            on (admin-only).
        :param string stateful: Whether the security group is stateful or not.

        :returns: A ``openstack.network.v2.security_group.SecurityGroup``
            representing the new security group.

        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        :raises: OpenStackCloudUnavailableFeature if security groups are
            not supported on this cloud.
        """

        # Security groups not supported
        if not self._has_secgroups():
            raise exc.OpenStackCloudUnavailableFeature(
                "Unavailable feature: security groups"
            )

        data = []
        security_group_json = {'name': name, 'description': description}
        if stateful is not None:
            security_group_json['stateful'] = stateful
        if project_id is not None:
            security_group_json['tenant_id'] = project_id
        if self._use_neutron_secgroups():
            return self.network.create_security_group(**security_group_json)
        else:
            data = proxy._json_response(
                self.compute.post(
                    '/os-security-groups',
                    json={'security_group': security_group_json},
                )
            )
        return self._normalize_secgroup(
            self._get_and_munchify('security_group', data)
        )

    def delete_security_group(self, name_or_id):
        """Delete a security group

        :param string name_or_id: The name or unique ID of the security group.

        :returns: True if delete succeeded, False otherwise.

        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        :raises: OpenStackCloudUnavailableFeature if security groups are
            not supported on this cloud.
        """
        # Security groups not supported
        if not self._has_secgroups():
            raise exc.OpenStackCloudUnavailableFeature(
                "Unavailable feature: security groups"
            )

        # TODO(mordred): Let's come back and stop doing a GET before we do
        #                the delete.
        secgroup = self.get_security_group(name_or_id)
        if secgroup is None:
            self.log.debug(
                'Security group %s not found for deleting', name_or_id
            )
            return False

        if self._use_neutron_secgroups():
            self.network.delete_security_group(
                secgroup['id'], ignore_missing=False
            )
            return True

        else:
            proxy._json_response(
                self.compute.delete(
                    '/os-security-groups/{id}'.format(id=secgroup['id'])
                )
            )
            return True

    @_utils.valid_kwargs('name', 'description', 'stateful')
    def update_security_group(self, name_or_id, **kwargs):
        """Update a security group

        :param string name_or_id: Name or ID of the security group to update.
        :param string name: New name for the security group.
        :param string description: New description for the security group.

        :returns: A ``openstack.network.v2.security_group.SecurityGroup``
            describing the updated security group.
        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        """
        # Security groups not supported
        if not self._has_secgroups():
            raise exc.OpenStackCloudUnavailableFeature(
                "Unavailable feature: security groups"
            )

        group = self.get_security_group(name_or_id)

        if group is None:
            raise exceptions.SDKException(
                "Security group %s not found." % name_or_id
            )

        if self._use_neutron_secgroups():
            return self.network.update_security_group(group['id'], **kwargs)
        else:
            for key in ('name', 'description'):
                kwargs.setdefault(key, group[key])
            data = proxy._json_response(
                self.compute.put(
                    '/os-security-groups/{id}'.format(id=group['id']),
                    json={'security_group': kwargs},
                )
            )
        return self._normalize_secgroup(
            self._get_and_munchify('security_group', data)
        )

    def create_security_group_rule(
        self,
        secgroup_name_or_id,
        port_range_min=None,
        port_range_max=None,
        protocol=None,
        remote_ip_prefix=None,
        remote_group_id=None,
        remote_address_group_id=None,
        direction='ingress',
        ethertype='IPv4',
        project_id=None,
        description=None,
    ):
        """Create a new security group rule

        :param string secgroup_name_or_id:
            The security group name or ID to associate with this security
            group rule. If a non-unique group name is given, an exception
            is raised.
        :param int port_range_min:
            The minimum port number in the range that is matched by the
            security group rule. If the protocol is TCP or UDP, this value
            must be less than or equal to the port_range_max attribute value.
            If nova is used by the cloud provider for security groups, then
            a value of None will be transformed to -1.
        :param int port_range_max:
            The maximum port number in the range that is matched by the
            security group rule. The port_range_min attribute constrains the
            port_range_max attribute. If nova is used by the cloud provider
            for security groups, then a value of None will be transformed
            to -1.
        :param string protocol:
            The protocol that is matched by the security group rule. Valid
            values are None, tcp, udp, and icmp.
        :param string remote_ip_prefix:
            The remote IP prefix to be associated with this security group
            rule. This attribute matches the specified IP prefix as the
            source IP address of the IP packet.
        :param string remote_group_id:
            The remote group ID to be associated with this security group
            rule.
        :param string remote_address_group_id:
            The remote address group ID to be associated with this security
            group rule.
        :param string direction:
            Ingress or egress: The direction in which the security group
            rule is applied. For a compute instance, an ingress security
            group rule is applied to incoming (ingress) traffic for that
            instance. An egress rule is applied to traffic leaving the
            instance.
        :param string ethertype:
            Must be IPv4 or IPv6, and addresses represented in CIDR must
            match the ingress or egress rules.
        :param string project_id:
            Specify the project ID this security group will be created
            on (admin-only).
        :param string description:
            Description of the rule, max 255 characters.

        :returns: A ``openstack.network.v2.security_group.SecurityGroup``
            representing the new security group rule.
        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        """
        # Security groups not supported
        if not self._has_secgroups():
            raise exc.OpenStackCloudUnavailableFeature(
                "Unavailable feature: security groups"
            )

        secgroup = self.get_security_group(secgroup_name_or_id)
        if not secgroup:
            raise exceptions.SDKException(
                "Security group %s not found." % secgroup_name_or_id
            )

        if self._use_neutron_secgroups():
            # NOTE: Nova accepts -1 port numbers, but Neutron accepts None
            # as the equivalent value.
            rule_def = {
                'security_group_id': secgroup['id'],
                'port_range_min': None
                if port_range_min == -1
                else port_range_min,
                'port_range_max': None
                if port_range_max == -1
                else port_range_max,
                'protocol': protocol,
                'remote_ip_prefix': remote_ip_prefix,
                'remote_group_id': remote_group_id,
                'remote_address_group_id': remote_address_group_id,
                'direction': direction,
                'ethertype': ethertype,
            }
            if project_id is not None:
                rule_def['tenant_id'] = project_id
            if description is not None:
                rule_def["description"] = description
            return self.network.create_security_group_rule(**rule_def)
        else:
            # NOTE: Neutron accepts None for protocol. Nova does not.
            if protocol is None:
                raise exceptions.SDKException('Protocol must be specified')

            if direction == 'egress':
                self.log.debug(
                    'Rule creation failed: Nova does not support egress rules'
                )
                raise exceptions.SDKException('No support for egress rules')

            # NOTE: Neutron accepts None for ports, but Nova requires -1
            # as the equivalent value for ICMP.
            #
            # For TCP/UDP, if both are None, Neutron allows this and Nova
            # represents this as all ports (1-65535). Nova does not accept
            # None values, so to hide this difference, we will automatically
            # convert to the full port range. If only a single port value is
            # specified, it will error as normal.
            if protocol == 'icmp':
                if port_range_min is None:
                    port_range_min = -1
                if port_range_max is None:
                    port_range_max = -1
            elif protocol in ['tcp', 'udp']:
                if port_range_min is None and port_range_max is None:
                    port_range_min = 1
                    port_range_max = 65535

            security_group_rule_dict = dict(
                security_group_rule=dict(
                    parent_group_id=secgroup['id'],
                    ip_protocol=protocol,
                    from_port=port_range_min,
                    to_port=port_range_max,
                    cidr=remote_ip_prefix,
                    group_id=remote_group_id,
                )
            )
            if project_id is not None:
                security_group_rule_dict['security_group_rule'][
                    'tenant_id'
                ] = project_id
            data = proxy._json_response(
                self.compute.post(
                    '/os-security-group-rules', json=security_group_rule_dict
                )
            )
        return self._normalize_secgroup_rule(
            self._get_and_munchify('security_group_rule', data)
        )

    def delete_security_group_rule(self, rule_id):
        """Delete a security group rule

        :param string rule_id: The unique ID of the security group rule.

        :returns: True if delete succeeded, False otherwise.

        :raises: :class:`~openstack.exceptions.SDKException` on operation
            error.
        :raises: OpenStackCloudUnavailableFeature if security groups are
            not supported on this cloud.
        """
        # Security groups not supported
        if not self._has_secgroups():
            raise exc.OpenStackCloudUnavailableFeature(
                "Unavailable feature: security groups"
            )

        if self._use_neutron_secgroups():
            self.network.delete_security_group_rule(
                rule_id, ignore_missing=False
            )
            return True

        else:
            try:
                exceptions.raise_from_response(
                    self.compute.delete(
                        '/os-security-group-rules/{id}'.format(id=rule_id)
                    )
                )
            except exceptions.NotFoundException:
                return False

            return True

    def _has_secgroups(self):
        if not self.secgroup_source:
            return False
        else:
            return self.secgroup_source.lower() in ('nova', 'neutron')

    def _use_neutron_secgroups(self):
        return (
            self.has_service('network') and self.secgroup_source == 'neutron'
        )

    def _normalize_secgroups(self, groups):
        """Normalize the structure of security groups

        This makes security group dicts, as returned from nova, look like the
        security group dicts as returned from neutron. This does not make them
        look exactly the same, but it's pretty close.

        :param list groups: A list of security group dicts.

        :returns: A list of normalized dicts.
        """
        ret = []
        for group in groups:
            ret.append(self._normalize_secgroup(group))
        return ret

    # TODO(stephenfin): Remove this once we get rid of support for nova
    # secgroups
    def _normalize_secgroup(self, group):
        ret = utils.Munch()
        # Copy incoming group because of shared dicts in unittests
        group = group.copy()

        # Discard noise
        self._remove_novaclient_artifacts(group)

        rules = self._normalize_secgroup_rules(
            group.pop('security_group_rules', group.pop('rules', []))
        )
        project_id = group.pop('tenant_id', '')
        project_id = group.pop('project_id', project_id)

        ret['location'] = self._get_current_location(project_id=project_id)
        ret['id'] = group.pop('id')
        ret['name'] = group.pop('name')
        ret['security_group_rules'] = rules
        ret['description'] = group.pop('description')
        ret['properties'] = group

        if self._use_neutron_secgroups():
            ret['stateful'] = group.pop('stateful', True)

        # Backwards compat with Neutron
        if not self.strict_mode:
            ret['tenant_id'] = project_id
            ret['project_id'] = project_id
            for key, val in ret['properties'].items():
                ret.setdefault(key, val)

        return ret

    # TODO(stephenfin): Remove this once we get rid of support for nova
    # secgroups
    def _normalize_secgroup_rules(self, rules):
        """Normalize the structure of nova security group rules

        Note that nova uses -1 for non-specific port values, but neutron
        represents these with None.

        :param list rules: A list of security group rule dicts.

        :returns: A list of normalized dicts.
        """
        ret = []
        for rule in rules:
            ret.append(self._normalize_secgroup_rule(rule))
        return ret

    # TODO(stephenfin): Remove this once we get rid of support for nova
    # secgroups
    def _normalize_secgroup_rule(self, rule):
        ret = utils.Munch()
        # Copy incoming rule because of shared dicts in unittests
        rule = rule.copy()

        ret['id'] = rule.pop('id')
        ret['direction'] = rule.pop('direction', 'ingress')
        ret['ethertype'] = rule.pop('ethertype', 'IPv4')
        port_range_min = rule.get(
            'port_range_min', rule.pop('from_port', None)
        )
        if port_range_min == -1:
            port_range_min = None
        if port_range_min is not None:
            port_range_min = int(port_range_min)
        ret['port_range_min'] = port_range_min
        port_range_max = rule.pop('port_range_max', rule.pop('to_port', None))
        if port_range_max == -1:
            port_range_max = None
        if port_range_min is not None:
            port_range_min = int(port_range_min)
        ret['port_range_max'] = port_range_max
        ret['protocol'] = rule.pop('protocol', rule.pop('ip_protocol', None))
        ret['remote_ip_prefix'] = rule.pop(
            'remote_ip_prefix', rule.pop('ip_range', {}).get('cidr', None)
        )
        ret['security_group_id'] = rule.pop(
            'security_group_id', rule.pop('parent_group_id', None)
        )
        ret['remote_group_id'] = rule.pop('remote_group_id', None)
        project_id = rule.pop('tenant_id', '')
        project_id = rule.pop('project_id', project_id)
        ret['location'] = self._get_current_location(project_id=project_id)
        ret['properties'] = rule

        # Backwards compat with Neutron
        if not self.strict_mode:
            ret['tenant_id'] = project_id
            ret['project_id'] = project_id
            for key, val in ret['properties'].items():
                ret.setdefault(key, val)
        return ret

    def _remove_novaclient_artifacts(self, item):
        # Remove novaclient artifacts
        item.pop('links', None)
        item.pop('NAME_ATTR', None)
        item.pop('HUMAN_ID', None)
        item.pop('human_id', None)
        item.pop('request_ids', None)
        item.pop('x_openstack_request_ids', None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.database.v1 import instance
from openstack.tests.unit import base

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'flavor': '1',
    'id': IDENTIFIER,
    'links': '3',
    'name': '4',
    'status': '5',
    'volume': '6',
    'datastore': {'7': 'seven'},
    'region': '8',
    'hostname': '9',
    'created': '10',
    'updated': '11',
}


class TestInstance(base.TestCase):
    def test_basic(self):
        sot = instance.Instance()
        self.assertEqual('instance', sot.resource_key)
        self.assertEqual('instances', sot.resources_key)
        self.assertEqual('/instances', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = instance.Instance(**EXAMPLE)
        self.assertEqual(EXAMPLE['flavor'], sot.flavor)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['links'], sot.links)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['volume'], sot.volume)
        self.assertEqual(EXAMPLE['datastore'], sot.datastore)
        self.assertEqual(EXAMPLE['region'], sot.region)
        self.assertEqual(EXAMPLE['hostname'], sot.hostname)
        self.assertEqual(EXAMPLE['created'], sot.created_at)
        self.assertEqual(EXAMPLE['updated'], sot.updated_at)

    def test_enable_root_user(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.body = {'user': {'name': 'root', 'password': 'foo'}}
        response.json = mock.Mock(return_value=response.body)
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)

        self.assertEqual(response.body['user'], sot.enable_root_user(sess))

        url = "instances/%s/root" % IDENTIFIER
        sess.post.assert_called_with(
            url,
        )

    def test_is_root_enabled(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.body = {'rootEnabled': True}
        response.json = mock.Mock(return_value=response.body)
        sess = mock.Mock()
        sess.get = mock.Mock(return_value=response)

        self.assertTrue(sot.is_root_enabled(sess))

        url = "instances/%s/root" % IDENTIFIER
        sess.get.assert_called_with(
            url,
        )

    def test_action_restart(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)

        self.assertIsNone(sot.restart(sess))

        url = "instances/%s/action" % IDENTIFIER
        body = {'restart': None}
        sess.post.assert_called_with(url, json=body)

    def test_action_resize(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)
        flavor = 'http://flavor/flav'

        self.assertIsNone(sot.resize(sess, flavor))

        url = "instances/%s/action" % IDENTIFIER
        body = {'resize': {'flavorRef': flavor}}
        sess.post.assert_called_with(url, json=body)

    def test_action_resize_volume(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)
        size = 4

        self.assertIsNone(sot.resize_volume(sess, size))

        url = "instances/%s/action" % IDENTIFIER
        body = {'resize': {'volume': size}}
        sess.post.assert_called_with(url, json=body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import utils

CONSOLE_TYPE_PROTOCOL_MAPPING = {
    'novnc': 'vnc',
    'xvpvnc': 'vnc',
    'spice-html5': 'spice',
    'rdp-html5': 'rdp',
    'serial': 'serial',
    'webmks': 'mks',
}


class ServerRemoteConsole(resource.Resource):
    resource_key = 'remote_console'
    base_path = '/servers/%(server_id)s/remote-consoles'

    # capabilities
    allow_create = True
    allow_fetch = False
    allow_commit = False
    allow_delete = False
    allow_list = False

    _max_microversion = '2.8'

    #: Protocol of the remote console.
    protocol = resource.Body('protocol')
    #: Type of the remote console.
    type = resource.Body('type')
    #: URL used to connect to the console.
    url = resource.Body('url')
    #: The ID for the server.
    server_id = resource.URI('server_id')

    def create(self, session, prepend_key=True, base_path=None, **params):
        if not self.protocol:
            self.protocol = CONSOLE_TYPE_PROTOCOL_MAPPING.get(self.type)
        if (
            not utils.supports_microversion(session, '2.8')
            and self.type == 'webmks'
        ):
            raise ValueError(
                'Console type webmks is not supported on server side'
            )
        return super(ServerRemoteConsole, self).create(
            session, prepend_key=prepend_key, base_path=base_path, **params
        )
